"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import release_policy_pb2
import secret_store_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _DataRoomStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DataRoomStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DataRoomStatus.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    Active: _DataRoomStatus.ValueType  # 0
    Stopped: _DataRoomStatus.ValueType  # 1

class DataRoomStatus(_DataRoomStatus, metaclass=_DataRoomStatusEnumTypeWrapper): ...

Active: DataRoomStatus.ValueType  # 0
Stopped: DataRoomStatus.ValueType  # 1
global___DataRoomStatus = DataRoomStatus

@typing_extensions.final
class ArchV2Request(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CREATEPOLICY_FIELD_NUMBER: builtins.int
    SECRETSTORE_FIELD_NUMBER: builtins.int
    UPDATESTATUS_FIELD_NUMBER: builtins.int
    GETVERIFICATIONKEY_FIELD_NUMBER: builtins.int
    CREATEDATAROOM_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOM_FIELD_NUMBER: builtins.int
    @property
    def createPolicy(self) -> global___CreatePolicyRequest: ...
    @property
    def secretStore(self) -> secret_store_pb2.SecretStoreRequest: ...
    @property
    def updateStatus(self) -> global___UpdateDataRoomStatusRequest: ...
    @property
    def getVerificationKey(self) -> global___GetVerificationKeyRequest: ...
    @property
    def createDataRoom(self) -> global___CreateDataRoomRequest:
        """TODO: Remove this once we can create DCRs with api-platform"""
    @property
    def retrieveDataRoom(self) -> global___RetrieveDataRoomRequest: ...
    def __init__(
        self,
        *,
        createPolicy: global___CreatePolicyRequest | None = ...,
        secretStore: secret_store_pb2.SecretStoreRequest | None = ...,
        updateStatus: global___UpdateDataRoomStatusRequest | None = ...,
        getVerificationKey: global___GetVerificationKeyRequest | None = ...,
        createDataRoom: global___CreateDataRoomRequest | None = ...,
        retrieveDataRoom: global___RetrieveDataRoomRequest | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["createDataRoom", b"createDataRoom", "createPolicy", b"createPolicy", "getVerificationKey", b"getVerificationKey", "request", b"request", "retrieveDataRoom", b"retrieveDataRoom", "secretStore", b"secretStore", "updateStatus", b"updateStatus"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["createDataRoom", b"createDataRoom", "createPolicy", b"createPolicy", "getVerificationKey", b"getVerificationKey", "request", b"request", "retrieveDataRoom", b"retrieveDataRoom", "secretStore", b"secretStore", "updateStatus", b"updateStatus"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["request", b"request"]) -> typing_extensions.Literal["createPolicy", "secretStore", "updateStatus", "getVerificationKey", "createDataRoom", "retrieveDataRoom"] | None: ...

global___ArchV2Request = ArchV2Request

@typing_extensions.final
class ArchV2Response(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FAILURE_FIELD_NUMBER: builtins.int
    CREATEPOLICY_FIELD_NUMBER: builtins.int
    SECRETSTORE_FIELD_NUMBER: builtins.int
    UPDATESTATUS_FIELD_NUMBER: builtins.int
    GETVERIFICATIONKEY_FIELD_NUMBER: builtins.int
    CREATEDATAROOM_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOM_FIELD_NUMBER: builtins.int
    failure: builtins.str
    @property
    def createPolicy(self) -> global___CreatePolicyResponse: ...
    @property
    def secretStore(self) -> secret_store_pb2.SecretStoreResponse: ...
    @property
    def updateStatus(self) -> global___UpdateDataRoomStatusResponse: ...
    @property
    def getVerificationKey(self) -> global___GetVerificationKeyResponse: ...
    @property
    def createDataRoom(self) -> global___CreateDataRoomResponse: ...
    @property
    def retrieveDataRoom(self) -> global___RetrieveDataRoomResponse: ...
    def __init__(
        self,
        *,
        failure: builtins.str = ...,
        createPolicy: global___CreatePolicyResponse | None = ...,
        secretStore: secret_store_pb2.SecretStoreResponse | None = ...,
        updateStatus: global___UpdateDataRoomStatusResponse | None = ...,
        getVerificationKey: global___GetVerificationKeyResponse | None = ...,
        createDataRoom: global___CreateDataRoomResponse | None = ...,
        retrieveDataRoom: global___RetrieveDataRoomResponse | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["createDataRoom", b"createDataRoom", "createPolicy", b"createPolicy", "failure", b"failure", "getVerificationKey", b"getVerificationKey", "response", b"response", "retrieveDataRoom", b"retrieveDataRoom", "secretStore", b"secretStore", "updateStatus", b"updateStatus"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["createDataRoom", b"createDataRoom", "createPolicy", b"createPolicy", "failure", b"failure", "getVerificationKey", b"getVerificationKey", "response", b"response", "retrieveDataRoom", b"retrieveDataRoom", "secretStore", b"secretStore", "updateStatus", b"updateStatus"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["response", b"response"]) -> typing_extensions.Literal["failure", "createPolicy", "secretStore", "updateStatus", "getVerificationKey", "createDataRoom", "retrieveDataRoom"] | None: ...

global___ArchV2Response = ArchV2Response

@typing_extensions.final
class GetVerificationKeyRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetVerificationKeyRequest = GetVerificationKeyRequest

@typing_extensions.final
class GetVerificationKeyResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VERIFICATIONKEY_FIELD_NUMBER: builtins.int
    verificationKey: builtins.bytes
    def __init__(
        self,
        *,
        verificationKey: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["verificationKey", b"verificationKey"]) -> None: ...

global___GetVerificationKeyResponse = GetVerificationKeyResponse

@typing_extensions.final
class CreateDataRoomRequest(google.protobuf.message.Message):
    """TODO: Remove this once we can create DCRs with api-platform"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOM_FIELD_NUMBER: builtins.int
    DATAROOMMETADATA_FIELD_NUMBER: builtins.int
    dataRoom: builtins.str
    dataRoomMetadata: builtins.bytes
    def __init__(
        self,
        *,
        dataRoom: builtins.str = ...,
        dataRoomMetadata: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_dataRoomMetadata", b"_dataRoomMetadata", "dataRoomMetadata", b"dataRoomMetadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_dataRoomMetadata", b"_dataRoomMetadata", "dataRoom", b"dataRoom", "dataRoomMetadata", b"dataRoomMetadata"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_dataRoomMetadata", b"_dataRoomMetadata"]) -> typing_extensions.Literal["dataRoomMetadata"] | None: ...

global___CreateDataRoomRequest = CreateDataRoomRequest

@typing_extensions.final
class CreateDataRoomResponse(google.protobuf.message.Message):
    """TODO: Remove this once we can create DCRs with api-platform"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    DATAROOMVALIDATIONERROR_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    @property
    def dataRoomValidationError(self) -> global___DataRoomValidationError: ...
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
        dataRoomValidationError: global___DataRoomValidationError | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId", "dataRoomValidationError", b"dataRoomValidationError", "response", b"response"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId", "dataRoomValidationError", b"dataRoomValidationError", "response", b"response"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["response", b"response"]) -> typing_extensions.Literal["dataRoomId", "dataRoomValidationError"] | None: ...

global___CreateDataRoomResponse = CreateDataRoomResponse

@typing_extensions.final
class CreatePolicyRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POLICY_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    @property
    def policy(self) -> release_policy_pb2.ReleasePolicy: ...
    scope: builtins.bytes
    """/ Required if referencing a dataset as part of the policy"""
    def __init__(
        self,
        *,
        policy: release_policy_pb2.ReleasePolicy | None = ...,
        scope: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_scope", b"_scope", "policy", b"policy", "scope", b"scope"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_scope", b"_scope", "policy", b"policy", "scope", b"scope"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_scope", b"_scope"]) -> typing_extensions.Literal["scope"] | None: ...

global___CreatePolicyRequest = CreatePolicyRequest

@typing_extensions.final
class CreatePolicyResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POLICYID_FIELD_NUMBER: builtins.int
    policyId: builtins.str
    def __init__(
        self,
        *,
        policyId: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["policyId", b"policyId"]) -> None: ...

global___CreatePolicyResponse = CreatePolicyResponse

@typing_extensions.final
class UpdateDataRoomStatusRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.str
    status: global___DataRoomStatus.ValueType
    def __init__(
        self,
        *,
        dataRoomId: builtins.str = ...,
        status: global___DataRoomStatus.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId", "status", b"status"]) -> None: ...

global___UpdateDataRoomStatusRequest = UpdateDataRoomStatusRequest

@typing_extensions.final
class UpdateDataRoomStatusResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UpdateDataRoomStatusResponse = UpdateDataRoomStatusResponse

@typing_extensions.final
class RetrieveDataRoomRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId"]) -> None: ...

global___RetrieveDataRoomRequest = RetrieveDataRoomRequest

@typing_extensions.final
class RetrieveDataRoomResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOM_FIELD_NUMBER: builtins.int
    dataRoom: builtins.str
    """/ JSON string of the HL representation"""
    def __init__(
        self,
        *,
        dataRoom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoom", b"dataRoom"]) -> None: ...

global___RetrieveDataRoomResponse = RetrieveDataRoomResponse

@typing_extensions.final
class DataRoomValidationError(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MESSAGE_FIELD_NUMBER: builtins.int
    PERMISSIONINDEX_FIELD_NUMBER: builtins.int
    COMPUTENODEID_FIELD_NUMBER: builtins.int
    USERPERMISSIONID_FIELD_NUMBER: builtins.int
    ATTESTATIONSPECIFICATIONID_FIELD_NUMBER: builtins.int
    AUTHENTICATIONMETHODID_FIELD_NUMBER: builtins.int
    message: builtins.str
    permissionIndex: builtins.int
    computeNodeId: builtins.str
    userPermissionId: builtins.str
    attestationSpecificationId: builtins.str
    authenticationMethodId: builtins.str
    def __init__(
        self,
        *,
        message: builtins.str = ...,
        permissionIndex: builtins.int | None = ...,
        computeNodeId: builtins.str | None = ...,
        userPermissionId: builtins.str | None = ...,
        attestationSpecificationId: builtins.str | None = ...,
        authenticationMethodId: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_attestationSpecificationId", b"_attestationSpecificationId", "_authenticationMethodId", b"_authenticationMethodId", "_computeNodeId", b"_computeNodeId", "_permissionIndex", b"_permissionIndex", "_userPermissionId", b"_userPermissionId", "attestationSpecificationId", b"attestationSpecificationId", "authenticationMethodId", b"authenticationMethodId", "computeNodeId", b"computeNodeId", "permissionIndex", b"permissionIndex", "userPermissionId", b"userPermissionId"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_attestationSpecificationId", b"_attestationSpecificationId", "_authenticationMethodId", b"_authenticationMethodId", "_computeNodeId", b"_computeNodeId", "_permissionIndex", b"_permissionIndex", "_userPermissionId", b"_userPermissionId", "attestationSpecificationId", b"attestationSpecificationId", "authenticationMethodId", b"authenticationMethodId", "computeNodeId", b"computeNodeId", "message", b"message", "permissionIndex", b"permissionIndex", "userPermissionId", b"userPermissionId"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_attestationSpecificationId", b"_attestationSpecificationId"]) -> typing_extensions.Literal["attestationSpecificationId"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_authenticationMethodId", b"_authenticationMethodId"]) -> typing_extensions.Literal["authenticationMethodId"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_computeNodeId", b"_computeNodeId"]) -> typing_extensions.Literal["computeNodeId"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_permissionIndex", b"_permissionIndex"]) -> typing_extensions.Literal["permissionIndex"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_userPermissionId", b"_userPermissionId"]) -> typing_extensions.Literal["userPermissionId"] | None: ...

global___DataRoomValidationError = DataRoomValidationError



from __future__ import annotations

import io
import json
from typing import TYPE_CHECKING, Any, Dict, List, Optional
import zipfile

from decentriq_dcr_compiler.schemas import (
    EnvironmentComputationNode
)
from typing_extensions import Self

from ..session import Session
from .high_level_node import ContainerComputationNode
from .node_definitions import NodeDefinition

if TYPE_CHECKING:
    from ..client import Client
    
class PythonEnvironmentComputeNodeDefinition(NodeDefinition):
    """
    Class representing a Python Environment Definition.
    """
    
    def __init__(
        self,
        name: str,
        requirements_txt: str,
        id: Optional[str] = None,
    ) -> None:
        """
        Initialise a `PythonEnvironmentDefinition`.

        **Parameters**:
        - `name`: Name of the `PythonEnvironmentDefinition`.
        - `requirements_txt`: Content of the `requirements.txt` file which list the packages for the environment.
        """
        super().__init__(name, id=id or name)
        self.requirements_txt = requirements_txt
        self.scripting_specification_id = "decentriq.python-ml-worker-32-64"
        self.static_content_specification_id = "decentriq.driver"
        
    def _get_high_level_representation(self) -> Dict[str, str]:
        """
        Retrieve the high level representation of the `PythonEnvironmentDefinition`.
        """

        computation_node = {
            "id": self.id,
            "name": self.name,
            "kind": {
                "computation": {
                    "kind": {
                        "environment": {
                            "kind": {
                                "python": {
                                    "staticContentSpecificationId": self.static_content_specification_id,
                                    "scriptingSpecificationId": self.scripting_specification_id,
                                    "requirementsTxtContent": self.requirements_txt,
                                }
                            }
                        }
                    }
                }
            }
        }
        return computation_node

    @property
    def required_workers(self):
        return [self.scripting_specification_id, self.static_content_specification_id]

    @classmethod
    def _from_high_level(
        cls,
        id: str,
        name: str,
        node: EnvironmentComputationNode,
    ) -> Self:
        """
        Instantiate a `PythonEnvironmentComputeNode` from its high level representation.

        **Parameters**:
        - `name`: Name of the `PythonEnvironmentComputeNodeDefinition`.
        - `node`: Pydantic model of the `PythonEnvironmentComputeNode`.
        """
        environment_node = json.loads(node.model_dump_json())
        return cls(
            id=id,
            name=name,
            requirements_txt=environment_node["kind"]["python"]["requirementsTxtContent"],
        )

    def build(
        self,
        dcr_id: str,
        node_definition: NodeDefinition,
        client: Client,
        session: Session,
    ) -> PythonEnvironmentComputeNode:
        """
        Construct a PythonEnvironmentComputeNode from the Node Definition.

        **Parameters**:
        - `dcr_id`: ID of the DCR the node is a member of.
        - `node_definition`: Definition of the Python Environment Compute Node.
        - `client`: A `Client` object which can be used to perform operations such as uploading data
            and retrieving computation results.
        - `session`: The session with which to communicate with the enclave.
        """
        return PythonEnvironmentComputeNode(
            id=self.id,
            name=self.name,
            requirements_txt=self.requirements_txt,
            dcr_id=dcr_id,
            session=session,
            client=client,
            node_definition=node_definition,
        )

class PythonEnvironmentComputeNode(ContainerComputationNode):
    """
    Class representing an Environment.
    """

    def __init__(
        self,
        id: str,
        name: str,
        requirements_txt: str,
        dcr_id: str,
        session: Session,
        node_definition: NodeDefinition,
        *,
        client: Client,
    ) -> None:
        """
        Initialise an instance of a `PythonEnvironmentComputationNode`.

        **Parameters**:
        - `id`: ID of the `PythonEnvironmentComputeNode`.
        - `name`: Name of the `PythonEnvironmentComputeNode`.
        - `requirements_txt`: Content of the `requirements.txt` file which list the packages for the environment.
        - `dcr_id`: ID of the DCR the node is a member of.
        - `session`: The session with which to communicate with the enclave.
        - 'node_definition': Definition with which the node was built.
        - `client`: A `Client` object which can be used to perform operations such as uploading data
            and retrieving computation results.
        """
        super().__init__(
            name=name,
            client=client,
            session=session,
            dcr_id=dcr_id,
            id=id,
            definition=node_definition,
        )
        self.requirements_txt = requirements_txt
        
    def _get_computation_id(self) -> str:
        """
        Retrieve the ID of the node corresponding to the computation.
        """
        return f"{self.id}_env_create"

    def get_installation_report_as_dict(self) -> Optional[Dict[str, str]]:
        """
        Retrieve the virtual environment creation report to this `PythonEnvironmentComputeNode`.
        """
        report_node_id = f"{self.id}_env_report"
        result = self.session.run_computation_and_get_results(
            self.dcr_id, report_node_id, interval=1
        )
        if result:
            report = {}
            zip = zipfile.ZipFile(io.BytesIO(result), "r")
            if "report.json" in zip.namelist():
                report = json.loads(
                    zip.read("report.json").decode()
                )
            return report
        else:
            return None
    





# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: meta_sink.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'meta_sink.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fmeta_sink.proto\x12\tmeta_sink\"\x1a\n\nSingleFile\x12\x0c\n\x04name\x18\x01 \x01(\t\"\t\n\x07RawFile\"C\n\x07ZipFile\x12+\n\nsingleFile\x18\x01 \x01(\x0b\x32\x15.meta_sink.SingleFileH\x00\x42\x0b\n\tselection\"{\n\tSinkInput\x12\x12\n\ndependency\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12!\n\x03raw\x18\x03 \x01(\x0b\x32\x12.meta_sink.RawFileH\x00\x12!\n\x03zip\x18\x04 \x01(\x0b\x32\x12.meta_sink.ZipFileH\x00\x42\x06\n\x04\x66ile\"\xb4\x01\n\x1bMetaSinkWorkerConfiguration\x12#\n\x05input\x18\x01 \x01(\x0b\x32\x14.meta_sink.SinkInput\x12\x1d\n\x15\x61\x63\x63\x65ssTokenDependency\x18\x02 \x01(\t\x12\x13\n\x0b\x61\x64\x41\x63\x63ountId\x18\x03 \x01(\t\x12\x14\n\x0c\x61udienceName\x18\x04 \x01(\t\x12\x17\n\napiVersion\x18\x05 \x01(\tH\x00\x88\x01\x01\x42\r\n\x0b_apiVersionb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meta_sink_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SINGLEFILE']._serialized_start=30
  _globals['_SINGLEFILE']._serialized_end=56
  _globals['_RAWFILE']._serialized_start=58
  _globals['_RAWFILE']._serialized_end=67
  _globals['_ZIPFILE']._serialized_start=69
  _globals['_ZIPFILE']._serialized_end=136
  _globals['_SINKINPUT']._serialized_start=138
  _globals['_SINKINPUT']._serialized_end=261
  _globals['_METASINKWORKERCONFIGURATION']._serialized_start=264
  _globals['_METASINKWORKERCONFIGURATION']._serialized_end=444
# @@protoc_insertion_point(module_scope)



from enum import Enum


class ScriptingLanguage(str, Enum):
    python = "python"
    r = "r"


class FileContent:
    def __init__(self, name: str, content: str) -> None:
        self.name = name
        self.content = content


class Script(FileContent):
    def __init__(self, name: str, content: str, language: ScriptingLanguage) -> None:
        super().__init__(name, content)
        self.language = language


class PythonScript(Script):
    """
    Class representing a Python script.
    """

    def __init__(self, name: str, content: str) -> None:
        super().__init__(name, content, ScriptingLanguage.python)


class RScript(Script):
    """
    Class representing an R script.
    """

    def __init__(self, name: str, content: str) -> None:
        super().__init__(name, content, ScriptingLanguage.r)



from google.protobuf.json_format import MessageToDict

from ...proto import (
    ComputeNodeFormat,
    parse_length_delimited,
    serialize_length_delimited,
)
from ..node import Node
from .proto import DataSourceSnowflakeWorkerConfiguration, SnowflakeSource

__docformat__ = "restructuredtext"
__pdoc__ = {
    "proto": False,
}


class DataSourceSnowflake(Node):
    """
    Compute node that fetches a dataset from a Snowflake stage.
    """

    def __init__(
        self,
        name: str,
        warehouseName: str,
        databaseName: str,
        schemaName: str,
        tableName: str,
        stageName: str,
        credentials_dependency: str,
    ) -> None:
        config = DataSourceSnowflakeWorkerConfiguration(
            source=SnowflakeSource(
                warehouseName=warehouseName,
                databaseName=databaseName,
                schemaName=schemaName,
                tableName=tableName,
                stageName=stageName,
            ),
            credentialsDependency=credentials_dependency,
        )
        config_serialized = serialize_length_delimited(config)
        super().__init__(
            name,
            config=config_serialized,
            enclave_type="decentriq.data-source-snowflake-worker",
            dependencies=[credentials_dependency],
            output_format=ComputeNodeFormat.ZIP,
        )


class DataSourceSnowflakeWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = DataSourceSnowflakeWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


__all__ = [
    "DataSourceSnowflake",
]



from typing import TYPE_CHECKING, Any, Callable, List, TypeVar, Protocol, cast

from .api import Api, ApiError
from .authentication import Auth
from .channel import Channel, CompilerRequest, CompilerResponse
from .graphql import GqlClient
from .proto.attestation_pb2 import AttestationSpecification
from .proto.gcg_pb2 import GcgRequest, GcgRequestV2, GcgResponse, GcgResponseV2

Response = TypeVar("Response")


class Connection:
    """
    Class that wraps a Channel and re-creates the enclave connection in case of exceptions
    """
    channel: Channel
    driver_attestation_specification: AttestationSpecification
    api: Api
    graphql_api: GqlClient
    unsafe_disable_known_root_ca_check: bool
    max_retries: int

    def __init__(
        self,
        driver_attestation_specification: AttestationSpecification,
        api: Api,
        graphql_api: GqlClient,
        unsafe_disable_known_root_ca_check: bool = False,
        max_retries: int = 5,
    ) -> None:
        self.driver_attestation_specification = driver_attestation_specification
        self.api = api
        self.graphql_api = graphql_api
        self.unsafe_disable_known_root_ca_check = unsafe_disable_known_root_ca_check
        self.channel = Channel(
            self.driver_attestation_specification,
            self.api,
            self.graphql_api,
            self.unsafe_disable_known_root_ca_check,
        )
        self.max_retries = max_retries

    def _retry_request(
        self, request_call: Callable[..., Response], *args: Any
    ) -> Response:
        for _ in range(0, self.max_retries):
            try:
                response = request_call(self.channel, *args)
                return response
            except ApiError as api_error:
                if api_error.message == "enclave unavailable":
                    self.channel = Channel(
                        self.driver_attestation_specification,
                        self.api,
                        self.graphql_api,
                        self.unsafe_disable_known_root_ca_check,
                    )
                    continue
                elif api_error.message == "manual deletion":
                    raise Exception(
                        "Your session with the enclave has expired. Please report this issue to the Decentriq support team."
                    )
                else:
                    raise Exception(
                        f"Unexpected API error {api_error.message}. Please report this issue to the Decentriq support team."
                    )
            except Exception as e:
                raise e
        raise Exception("Maximum retry limit reached")

    def send_request_v2(
        self,
        request: GcgRequestV2,
    ) -> GcgResponseV2:
        return self._retry_request(Channel.send_request_v2, request)

    def send_request(
        self,
        request: GcgRequest,
        protocol: int,
        auth: Auth,
    ) -> List[GcgResponse]:
        return self._retry_request(Channel.send_request, request, protocol, auth)

    def send_request_raw(
        self,
        request: bytes,
        protocol: int,
    ) -> List[bytes]:
        return self._retry_request(
            Channel.send_request_raw, request, protocol
        )

    def send_compilable_request(
        self,
        compile_request: Callable[[CompilerRequest, Channel], bytes],
        request: CompilerRequest,
        decompile_response: Callable[[List[bytes]], CompilerResponse],
        protocol: int,
    ) -> CompilerResponse:
        return cast(
            CompilerResponse,
            self._retry_request(
                Channel.send_compilable_request,
                compile_request,
                request,
                decompile_response,
                protocol,
            ),
        )



from __future__ import annotations

import io
from typing import BinaryIO


class GcsCredentials:
    def __init__(self, credentials_json: str) -> None:
        self.credentials_json = credentials_json

    def as_binary_io(self) -> BinaryIO:
        return io.BytesIO(self.credentials_json.encode())



import json
from typing import BinaryIO
import io


class AzureBlobStorageCredentials:
    def __init__(
        self,
        storage_account: str,
        storage_container: str,
        blob_name: str,
        sas_token: str,
    ) -> None:
        self.storage_account = storage_account
        self.storage_container = storage_container
        self.blob_name = blob_name
        self.sas_token = sas_token

    def as_binary_io(self) -> BinaryIO:
        credentials = {
            "storageAccount": self.storage_account,
            "storageContainer": self.storage_container,
            "blobName": self.blob_name,
            "sasToken": self.sas_token,
        }
        return io.BytesIO(json.dumps(credentials).encode())



from enum import Enum
from typing import Any, Dict, List, Optional, Union

from typing_extensions import TypedDict, TypeAlias

from .proto import AttestationSpecification
from .storage import Key

JSONType: TypeAlias = Union[Dict[str, "JSONType"], List["JSONType"], str, int, float, bool, None]

class JobId:
    """
    Class for identifying running or already run jobs.

    Objects of this class can be used to retrieve results for processed computations.
    """

    def __init__(self, job_id: str, compute_node_id: str):
        self.id = job_id
        """The identifier of the job that processed a particular computation."""

        self.compute_node_id = compute_node_id
        """The id of the computation that was processed."""


class ScopeTypes(str, Enum):
    DATASET = "DATASET"
    DCR_DATA = "DCR_DATA"


class UserResponse(TypedDict):
    id: str
    email: str


class UserCsrRequest(TypedDict):
    csrPem: str


class UserCsrResponse(TypedDict):
    certChainPem: str


class SystemCaResponse(TypedDict):
    rootCertificate: str


class CreateSessionRequest(TypedDict):
    attestationSpecificationHash: str


class SessionJsonResponse(TypedDict):
    sessionId: str
    attestationSpecificationHash: str


class FinalizeUpload(TypedDict):
    uploadId: str
    manifest: str
    name: str
    manifestHash: str
    chunks: List[str]
    scopeId: str


class ChunkWrapper(TypedDict):
    hash: str
    data: str


class UploadDescription(TypedDict):
    uploadId: str


class ChunkDescription(TypedDict):
    chunkHash: str


class DataRoomKind(str, Enum):
    EXPERT = "EXPERT"
    DATA_SCIENCE = "DATA_SCIENCE"
    MEDIA = "MEDIA"
    LOOKALIKE_MEDIA = "LOOKALIKE_MEDIA"
    MEDIA_INSIGHTS = "MEDIA_INSIGHTS"
    AB_MEDIA = "AB_MEDIA"
    MEDIA_V3 = "MEDIA_V3"


class Owner(TypedDict):
    email: str


class DataRoomDescription(TypedDict):
    """The identifier of the data room"""

    id: str
    """The title that was given to the data room"""
    title: str
    """The hex-encoded hash of the driver attestation specification"""
    driverAttestationHash: str
    """Whether the data room has been stopped"""
    isStopped: bool
    """When the data room was created"""
    createdAt: str
    """When the data room was last updated"""
    updatedAt: str
    """Email address of the data room owner"""
    owner: Owner
    """The kind of data room"""
    kind: DataRoomKind


class DatasetUsage(str, Enum):
    PUBLISHED = "PUBLISHED"
    TEST = "TEST"


class DatasetDescription(TypedDict):
    """
    This class includes information about an uploaded dataset
    """

    id: str
    """The identifier of this dataset"""
    manifestHash: str
    """The data set id as a hex-encoded string. This id is also called the manifest hash."""
    name: str
    """The name of this dataset"""
    description: str
    """An optional description"""
    createdAt: str
    """When the dataset was uploaded"""
    size: int
    """Size of the dataset"""
    usage: DatasetUsage
    """Usage"""
    encryptionKeySecretId: Optional[str]
    """Secret store entry id for the encryption key"""
    metadataSecretId: Optional[str]
    """Secret store entry id for the metadata"""


class SignatureResponse(TypedDict):
    type: str
    data: List[int]


class FatquoteResBody(TypedDict):
    fatquoteBase64: str


class DatasetManifestMetadata(TypedDict):
    name: str
    manifestHash: str
    chunks: List[str]


class EnclaveSpecification(TypedDict):
    """
    This class includes information about an enclave deployed in the platform.
    Please refer to `decentriq_platform.EnclaveSpecifications` for a detailed explanation.
    """

    name: str
    version: str

    proto: AttestationSpecification
    """The Protobuf object."""
    workerProtocols: List[int]
    """The worker protocol versions supported by the node"""
    decoder: Optional[Any]
    """
    Decoder object that can be used to decode the binary configs belonging
    to enclaves of this type.
    """
    clientProtocols: Optional[List[int]]
    """The client protocol versions supported by the node"""


class EnclaveSpecificationJson(TypedDict):
    name: str
    version: str
    spec: str


class EnclaveSpecificationResponse(TypedDict):
    attestationSpecs: List[EnclaveSpecificationJson]


class TcbComponent(TypedDict):
    svn: int


class Tcb(TypedDict):
    sgxtcbcomponents: List[TcbComponent]
    pcesvn: int


class TcbLevel(TypedDict):
    tcb: Tcb
    tcbStatus: str


class TcbInfo(TypedDict):
    version: int
    issueDate: str
    nextUpdate: str
    fmspc: str
    pceId: str
    tcbType: int
    tcbEvaluationDataNumber: int
    tcbLevels: List[TcbLevel]


class TcbInfoContainer(TypedDict):
    tcbInfo: TcbInfo
    signature: str


class IasResponse(TypedDict):
    isvEnclaveQuoteBody: str
    isvEnclaveQuoteStatus: str


class TestDataset(TypedDict):
    manifest_hash: str
    key: Key


class DryRunOptions(TypedDict):
    test_datasets: Dict[str, TestDataset]


# The matching ID specified by the user.
class MatchingId(str, Enum):
    """
    The type of Matching ID to use.

    Members:
        - STRING
        - EMAIL
        - HASHED_EMAIL
        - PHONE_NUMBER
        - HASHED_PHONE_NUMBER
    """

    STRING = "STRING"
    EMAIL = "EMAIL"
    HASHED_EMAIL = "HASHED_EMAIL"
    PHONE_NUMBER = "PHONE_NUMBER"
    HASHED_PHONE_NUMBER = "HASHED_PHONE_NUMBER"


# Internal Matching ID used by the SDK.
# Maps to the above user specified `MatchingId`.
class MatchingIdFormat(str, Enum):
    STRING = "STRING"
    EMAIL = "EMAIL"
    HASH_SHA256_HEX = "HASH_SHA256_HEX"
    PHONE_NUMBER_E164 = "PHONE_NUMBER_E164"


class TableColumnHashingAlgorithm(str, Enum):
    SHA256_HEX = "SHA256_HEX"


class DataLabDatasetType(Enum):
    EMBEDDINGS = 1
    DEMOGRAPHICS = 2
    MATCH = 3
    SEGMENTS = 4


class Dataset(TypedDict):
    id: str
    manifestHash: str
    name: str


class DataLabDataset(TypedDict):
    name: str
    dataset: Dataset


class DataLabDefinition(TypedDict):
    id: str
    name: str
    datasets: List[DataLabDataset]
    usersDataset: Dataset
    segmentsDataset: Dataset
    demographicsDataset: Dataset
    embeddingsDataset: Dataset
    statistics: str
    requireDemographicsDataset: bool
    requireEmbeddingsDataset: bool
    isValidated: bool
    numEmbeddings: int
    matchingIdFormat: MatchingIdFormat
    matchingIdHashingAlgorithm: TableColumnHashingAlgorithm
    validationComputeJobId: str
    statisticsComputeJobId: str
    jobsDriverAttestationHash: str
    highLevelRepresentationAsString: str
    createdAt: str
    updatedAt: str
    dropInvalidRows: bool


class DataLabListFilter(Enum):
    VALIDATED = 1  # List validated DataLabs
    UNVALIDATED = 2  # List un-validated DataLabs


class DataRoom(TypedDict):
    id: str
    title: str
    kind: DataRoomKind
    createdAt: str
    updatedAt: str
    owner: UserResponse


class CreateMediaComputeJobInput(TypedDict):
    publishedDataRoomId: str
    computeNodeName: str
    cacheKey: str
    jobType: str
    jobIdHex: str


class MediaComputeJobFilterInput(TypedDict):
    publishedDataRoomId: str
    jobType: str
    cacheKey: str


class MediaComputeJob(TypedDict):
    jobIdHex: str
    publishedDataRoomId: str
    computeNodeName: str
    jobType: str
    cacheKey: str
    createdAt: str


class PublishedDataset(TypedDict):
    leafId: str
    user: str
    timestamp: int
    datasetHash: bytes


class OverlapInsightsCacheKey(TypedDict):
    dataRoomId: str
    advertiserDatasetHash: Optional[str]
    publisherUsersDatasetHash: Optional[str]
    publisherSegmentsDatasetHash: Optional[str]
    publisherDemographicsDatasetHash: Optional[str]
    publisherEmbeddingsDatasetHash: Optional[str]
    publishedDatasets: List[PublishedDataset]


MATCHING_ID_INTERNAL_LOOKUP = {
    MatchingId.STRING: (MatchingIdFormat.STRING, None),
    MatchingId.EMAIL: (MatchingIdFormat.EMAIL, None),
    MatchingId.HASHED_EMAIL: (
        MatchingIdFormat.EMAIL,
        TableColumnHashingAlgorithm.SHA256_HEX,
    ),
    MatchingId.PHONE_NUMBER: (MatchingIdFormat.PHONE_NUMBER_E164, None),
    MatchingId.HASHED_PHONE_NUMBER: (
        MatchingIdFormat.PHONE_NUMBER_E164,
        TableColumnHashingAlgorithm.SHA256_HEX,
    ),
}

class OrganizationUser(TypedDict):
    id: str
    email: str



from .analytics_dcr import AnalyticsDcr, AnalyticsDcrDefinition
from .builder import AnalyticsDcrBuilder, ParticipantPermission
from .matching_compute_nodes import (
    MatchingComputeNode,
    MatchingComputeNodeConfig,
    MatchingComputeNodeDefinition,
)
from .node_definitions import *
from .preview_compute_nodes import PreviewComputeNode, PreviewComputeNodeDefinition
from .python_compute_nodes import PythonComputeNode, PythonComputeNodeDefinition
from .python_environment_compute_nodes import PythonEnvironmentComputeNode, PythonEnvironmentComputeNodeDefinition
from .r_compute_nodes import RComputeNode, RComputeNodeDefinition
from .raw_data_nodes import RawDataNode, RawDataNodeDefinition
from .s3_sink_compute_nodes import (
    S3Provider,
    S3SinkComputeNode,
    S3SinkComputeNodeDefinition,
)
from .script import FileContent, PythonScript, RScript, Script, ScriptingLanguage
from .sql_compute_nodes import SqlComputeNode, SqlComputeNodeDefinition
from .sql_helper import read_input_csv_file
from .sqlite_compute_nodes import SqliteComputeNode, SqliteComputeNodeDefinition
from .synthetic_compute_nodes import (
    MaskType,
    SyntheticDataComputeNode,
    SyntheticDataComputeNodeDefinition,
    SyntheticNodeColumn,
)
from .table_data_nodes import (
    Column,
    FormatType,
    PrimitiveType,
    TableDataNode,
    TableDataNodeDefinition,
)
from .dataset_sink_compute_nodes import (
    DatasetSinkComputationNode,
    DatasetSinkComputeNodeDefinition,
    SinkInputFormat,
)
from .version import DATA_SCIENCE_DCR_SUPPORTED_VERSION

__pdoc__ = {
    "builder": False,
    "commits": False,
    "compute_nodes": False,
    "data_nodes": False,
    "data_science_dcr": False,
    "existing_builder": False,
    "high_level_node": False,
    "script": False,
    "version": False,
    "analytics_dcr": False,
    "matching_compute_nodes": False,
    "node_definitions": False,
    "preview_compute_nodes": False,
    "python_compute_nodes": False,
    "r_compute_nodes": False,
    "raw_data_nodes": False,
    "s3_sink_compute_nodes": False,
    "sql_compute_nodes": False,
    "sql_helper": False,
    "sqlite_compute_nodes": False,
    "synthetic_compute_nodes": False,
    "table_data_nodes": False,
    "dataset_sink_compute_nodes": False,
}

__docformat__ = "restructuredtext"

__all__ = [
    # DCR
    "AnalyticsDcr",
    "AnalyticsDcrDefinition",
    "AnalyticsDcrBuilder",
    # Compute nodes
    "PythonScript",
    "PythonEnvironmentComputeNode",
    "PythonEnvironmentComputeNodeDefinition",
    "RScript",
    "RComputeNodeDefinition",
    "PythonComputeNode",
    "PythonComputeNodeDefinition",
    "RComputeNode",
    "RComputeNodeDefinition",
    "SqlComputeNode",
    "SqlComputeNodeDefinition",
    "SqliteComputeNode",
    "SqliteComputeNodeDefinition",
    "S3SinkComputeNode",
    "S3SinkComputeNodeDefinition",
    "S3Provider",
    "MatchingComputeNode",
    "MatchingComputeNodeDefinition",
    "MatchingComputeNodeConfig",
    "MatchingComputeNodeDefinition",
    "SyntheticDataComputeNode",
    "SyntheticDataComputeNodeDefinition",
    "SyntheticNodeColumn",
    "SyntheticDataComputeNodeDefinition",
    "MaskType",
    "PreviewComputeNode",
    "PreviewComputeNodeDefinition",
    "FileContent",
    "DatasetSinkComputeNodeDefinition",
    "DatasetSinkComputationNode",
    "SinkInputFormat",
    # Data nodes
    "RawDataNode",
    "RawDataNodeDefinition",
    "TableDataNode",
    "TableDataNodeDefinition",
    "Column",
    "PrimitiveType",
    "FormatType",
    "read_input_csv_file",
]



from typing import List
from typing_extensions import Self
from .audience_definitions import (
    AudienceFilters,
    AudienceCombinator,
    RuleBasedAudienceDefinition,
    AudienceStatus,
    AudienceDefinitions,
    audience_id_from_name,
    CombineOperator,
)


class RuleBasedAudienceBuilder:
    """
    Builder for constructing rule-based audience definitions.
    """

    def __init__(
        self, *, name: str, source_audience_name: str, audiences: AudienceDefinitions
    ) -> None:
        """
        Initialise the rule-based audience builder.

        **Parameters**:
        - `name`: Name of the rule-based audience.
        - `source_audience_name`: Name of the source audience that the rule-based audience will be built from.
        - `audiences`: Definitions of all existing audiences.
        """
        self.name = name
        self.source_audience_name = source_audience_name
        self.audiences = audiences
        self.make_available_to_publisher = False
        self.filters = None
        self.combinators = None

    def with_make_available_to_publisher(self) -> Self:
        """
        Make the rule-based audience available to the publisher.
        """
        self.make_available_to_publisher = True
        return self

    def with_filters(self, filters: AudienceFilters) -> Self:
        """
        Set the filters to be applied to the source audience.

        **Parameters**:
        - `filters`: Filters to be applied to the source audience.
        """
        self.filters = filters
        return self

    def with_combinator(self, combinators: List[AudienceCombinator]) -> Self:
        """
        Set the combinators to be applied to the audiences.
        This defines how multiple audiences can be combined.

        **Parameters**:
        - `combinators`: The list of combinators used to combine audiences.
        """
        self.combinators = combinators
        return self

    def build(self) -> RuleBasedAudienceDefinition:
        """
        Build the rule-based audience definition.
        """
        source_ref_id = audience_id_from_name(self.source_audience_name, self.audiences)
        return RuleBasedAudienceDefinition(
            name=self.name,
            source_ref_name=self.source_audience_name,
            source_ref_id=source_ref_id,
            status=(
                AudienceStatus.PUBLISHED
                if self.make_available_to_publisher
                else AudienceStatus.READY
            ),
            filters=self.filters,
            combinators=self.combinators,
        )


class AudienceCombinatorBuilder:
    """
    Builder for constructing an `AudienceCombinator`.
    """

    def __init__(
        self,
        *,
        operator: CombineOperator,
        source_audience_name: str,
        audiences: AudienceDefinitions
    ) -> None:
        """
        Initialise the audience combinator builder.

        **Parameters**:
        - `operator`: The operator used to combine audiences.
        - `source_audience_name`: Name of the source audience to combine with.
        - `audiences`: Definitions of all existing audiences.
        """
        self.operator = operator
        self.source_audience_name = source_audience_name
        self.audiences = audiences
        self.filters = None

    def with_filters(self, filters: AudienceFilters) -> Self:
        """
        Set the filters to be applied to the source audience.

        **Parameters**:
        - `filters`: Filters to be applied to the source audience.
        """
        self.filters = filters
        return self

    def build(self) -> AudienceCombinator:
        """
        Build the audience combinator.
        """
        source_audience_id = audience_id_from_name(
            self.source_audience_name, audiences=self.audiences
        )
        return AudienceCombinator(
            operator=self.operator,
            source_audience_name=self.source_audience_name,
            source_audience_id=source_audience_id,
            filters=self.filters,
        )



import json
import struct
from enum import IntFlag
from hashlib import sha256
from typing import List, Optional, Union

import asn1crypto.pem
import asn1crypto.x509
import cryptography
import OpenSSL
import pem as Pem
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives.asymmetric.utils import encode_dss_signature
from sgx_ias_structs import QuoteBody as _QuoteBody

from .proto import serialize_length_delimited
from .proto import (
    AttestationSpecification,
    AttestationSpecificationIntelDcap,
    AttestationSpecificationIntelDcapMrsigner,
    Fatquote,
    FatquoteDcap,
    FatquoteDcapMrsigner,
    FatquoteEpid,
)
from .types import IasResponse, Tcb, TcbInfoContainer, TcbLevel
from .logger import logger
from .certs import intel_sgx_ias_root_ca, intel_sgx_dcap_root_ca
from datetime import datetime
from decentriq_transparency_verification import verify_measurement_transparency


class QuoteBody:
    """Represents the body of the SGX IAS Quote"""

    def __init__(self, quote):
        if not isinstance(quote, _QuoteBody):
            raise Exception("QuoteBody must be converted from sgx_ias_structs repr.")
        self.version = quote.version
        self.sign_type = quote.sign_type
        self.qe_svn = quote.qe_svn
        self.pce_svn = quote.pce_svn
        self.xeid = quote.xeid
        self.epid_group_id = quote.epid_group_id
        self.basename = quote.basename
        self.reportdata = quote.reportdata
        self.cpusvn = quote.cpusvn
        self.mrenclave = quote.mrenclave
        self.mrsigner = quote.mrsigner
        self.isvprodid = quote.isvprodid
        self.isvsvn = quote.isvsvn
        self.flags = quote.flags

    def __str__(self):
        obj_dict = {
            k: bytes(v).hex() if isinstance(v, (list, bytes)) else v
            for (k, v) in self.__dict__.items()
        }
        return json.dumps(obj_dict, indent=4)

    @classmethod
    def from_base64_string(cls, data):
        return cls(_QuoteBody.from_base64_string(data))


class CertificateValidationError(Exception):
    """Raised when the certificate validation failed"""

    pass


class SignatureValidationError(Exception):
    """Raised when the signature validation failed"""

    pass


class MessageParsingError(Exception):
    """Raised when the json message parsing failed"""

    pass


class ISVEnclaveQuoteStatusError(Exception):
    """Raised when the isv enclave quote status is not in an accepted status"""

    pass


class EnclaveQuoteFlagsError(Exception):
    """Raised when the flags in quote body don't match expected values"""

    pass


class MeasurementMismatchError(Exception):
    """Raised when enclave measurement does not match expected value"""

    pass


class MrsignerMismatchError(Exception):
    """Raised when enclave signer (mrsigner) does not match expected value"""

    pass

class TransparencyVerificationError(Exception):
    """Raised when we couldn't verify that the enclave was published transparently."""

    pass

class Verification:
    class IASAttributeFlags(IntFlag):
        INIT = 1
        DEBUG = 2
        MODE64BIT = 4
        PROVISIONKEY = 8
        EINITTOKENKEY = 16

    def __init__(
        self,
        attestation_specification: AttestationSpecification,
    ):
        self.attestation_specification = attestation_specification
        self.check_known_root_ca = True

    # Warning: this is unsafe to do, only do this if you deliberately want to test fake roots (e.g. for mocked enclaves)
    def disable_known_root_ca_check(self):
        self.check_known_root_ca = False

    def _verify_epid(self, epid: FatquoteEpid) -> bytes:
        spec_type = self.attestation_specification.WhichOneof(
            "attestation_specification"
        )
        if spec_type != "intelEpid":
            raise Exception(
                f"Incompatible attestation specification, expected EPID, got {spec_type}"
            )
        spec_epid = self.attestation_specification.intelEpid
        root_x509 = OpenSSL.crypto.load_certificate(
            OpenSSL.crypto.FILETYPE_ASN1, spec_epid.iasRootCaDer
        )
        trust_store = OpenSSL.crypto.X509Store()
        trust_store.add_cert(root_x509)
        cert_x509 = OpenSSL.crypto.load_certificate(
            OpenSSL.crypto.FILETYPE_PEM, epid.iasCertificate
        )
        context = OpenSSL.crypto.X509StoreContext(trust_store, cert_x509)
        context.verify_certificate()

        # Step 2: verify message signature
        ias_root_cert = cryptography.x509.load_pem_x509_certificate(epid.iasCertificate)
        ias_root_cert.public_key().verify(
            epid.iasSignature,
            epid.iasResponseBody,
            cryptography.hazmat.primitives.asymmetric.padding.PKCS1v15(),
            cryptography.hazmat.primitives.hashes.SHA256(),
        )

        # Step 3: parse message and get quote body
        message_dic: IasResponse = json.loads(epid.iasResponseBody)
        quote_body_encoded = message_dic["isvEnclaveQuoteBody"]
        isv_enclave_quote_status = message_dic["isvEnclaveQuoteStatus"]
        quote_body = _QuoteBody.from_base64_string(quote_body_encoded)

        # Step 4: check isv enclave quote status
        if isv_enclave_quote_status == "OK":
            pass
        elif (
            spec_epid.acceptConfigurationNeeded
            and isv_enclave_quote_status == "CONFIGURATION_NEEDED"
        ):
            pass
        elif (
            spec_epid.acceptGroupOutOfDate
            and isv_enclave_quote_status == "GROUP_OUT_OF_DATE"
        ):
            pass
        else:
            raise ISVEnclaveQuoteStatusError(
                "Status " + isv_enclave_quote_status + " not accepted"
            )

        # Step 5: check enclave quote flags
        flags = quote_body.flags
        if not (flags & Verification.IASAttributeFlags.INIT):
            raise EnclaveQuoteFlagsError
        if not (flags & Verification.IASAttributeFlags.MODE64BIT):
            raise EnclaveQuoteFlagsError
        if flags & Verification.IASAttributeFlags.DEBUG:
            if spec_epid.acceptDebug:
                logger.warning(
                    "!!!WARNING!!! DEBUG quote is being accepted, quote is NOT to be trusted !!!WARNING!!!"
                )
            else:
                raise EnclaveQuoteFlagsError

        # Step 6: assert expected measurement
        measurement = bytes(quote_body.mrenclave)
        if not (spec_epid.mrenclave == measurement):
            raise MeasurementMismatchError(
                f"Wrong measurement, expected measurement: {spec_epid.mrenclave.hex()}, actual measurement: {measurement.hex()}."
            )

        return quote_body.reportdata

    def _get_cpusvn(self, tcb: Tcb) -> List[int]:
        cpusvn_array = [component["svn"] for component in tcb["sgxtcbcomponents"]]
        return cpusvn_array

    def _dcap_get_cert(self, quote: bytes):
        auth_len = int.from_bytes(quote[1012:1014], "little")

        cert = quote[1014 + auth_len + 2 + 4 :]
        return cert

    def _dcap_check_status(
        self, spec_dcap: AttestationSpecificationIntelDcap, status: str
    ):
        if status in ["UpToDate", "SWHardeningNeeded"]:
            return
        if not spec_dcap.acceptOutOfDate and status in [
            "OutOfDate",
            "OutOfDateConfigurationNeeded",
        ] or not spec_dcap.acceptConfigurationNeeded and status in [
            "ConfigurationNeeded",
            "OutOfDateConfigurationNeeded",
            "ConfigurationAndSWHardeningNeeded",
        ] or not spec_dcap.acceptRevoked and status in ["Revoked"]:
            raise Exception(f"TCB status {status} not accepted")
        return


    def _dcap_find_tcb_level(
        self, tcb_levels: List[TcbLevel], cpusvn: List[int], pcesvn: int
    ) -> Optional[TcbLevel]:
        for level in tcb_levels:
            tcb_cpusvn = self._get_cpusvn(level["tcb"])
            if all([a >= b for a, b in zip(cpusvn, tcb_cpusvn)]):
                if pcesvn >= level["tcb"]["pcesvn"]:
                    return level
        return None

    def _verify_dcap_inner(
        self,
        dcap: FatquoteDcap,
        spec_dcap: Union[
            AttestationSpecificationIntelDcap, AttestationSpecificationIntelDcapMrsigner
        ],
    ) -> bytes:
        # https://github.com/intel/SGXDataCenterAttestationPrimitives/blob/b6d6145c21e7a452f05838af24b09965ae9b6f10/QuoteGeneration/quote_wrapper/common/inc/sgx_quote_3.h#L177
        quote = dcap.dcapQuote
        certificate = self._dcap_get_cert(quote)
        pck_certs = Pem.parse(certificate)
        # Validate PCK Cert Chain
        root_x509 = OpenSSL.crypto.load_certificate(
            OpenSSL.crypto.FILETYPE_ASN1, spec_dcap.dcapRootCaDer
        )
        trust_store = OpenSSL.crypto.X509Store()
        trust_store.add_cert(root_x509)
        pck_x509 = OpenSSL.crypto.load_certificate(
            OpenSSL.crypto.FILETYPE_PEM, pck_certs[0].as_bytes()
        )
        intermediate_certs = map(
            lambda cert: OpenSSL.crypto.load_certificate(
                OpenSSL.crypto.FILETYPE_PEM, cert.as_bytes()
            ),
            pck_certs[1:],
        )
        context = OpenSSL.crypto.X509StoreContext(
            trust_store, pck_x509, list(intermediate_certs)
        )
        context.verify_certificate()

        # Verify that the QE REPORT signature
        cert = cryptography.x509.load_pem_x509_certificate(certificate)
        pubk = cert.public_key()
        if not isinstance(pubk, ec.EllipticCurvePublicKey):
            raise Exception("Expected EC public key")

        # Convert the signature from raw format to DSS format
        r = int.from_bytes(quote[948:980], "big")
        s = int.from_bytes(quote[980:1012], "big")
        signature = encode_dss_signature(r, s)

        pubk.verify(
            signature,
            quote[564:948],
            ec.ECDSA(hashes.SHA256())
        )

        flags = quote[96]
        if not (flags & Verification.IASAttributeFlags.INIT):
            raise EnclaveQuoteFlagsError
        if not (flags & Verification.IASAttributeFlags.MODE64BIT):
            raise EnclaveQuoteFlagsError
        if flags & Verification.IASAttributeFlags.DEBUG:
            if spec_dcap.acceptDebug:
                logger.warning(
                    "!!!WARNING!!! DEBUG quote is being accepted, quote is NOT to be trusted !!!WARNING!!!"
                )
            else:
                raise EnclaveQuoteFlagsError

        tcb_info_dic: TcbInfoContainer = json.loads(dcap.tcbInfo)
        tcb_info_body = bytes(
            json.dumps(tcb_info_dic["tcbInfo"], separators=(",", ":")), "utf-8"
        )
        tcb_info_sign = bytearray.fromhex(tcb_info_dic["signature"])

        # Verify TCB Sign Cert
        tcb_cert = OpenSSL.crypto.load_certificate(
            OpenSSL.crypto.FILETYPE_PEM, dcap.tcbSignCert
        )
        tcb_cert_context = OpenSSL.crypto.X509StoreContext(
            trust_store, tcb_cert, list(intermediate_certs)
        )
        tcb_cert_context.verify_certificate()

        # Verify signature over the tcb info body
        tcb_sign_cert = cryptography.x509.load_pem_x509_certificate(dcap.tcbSignCert)
        tcb_pubk = tcb_sign_cert.public_key()
        if not isinstance(tcb_pubk, ec.EllipticCurvePublicKey):
            raise Exception("Expected EC public key")

        # Convert the signature from raw format to DSS format
        r = int.from_bytes(tcb_info_sign[:32], "big")
        s = int.from_bytes(tcb_info_sign[32:], "big")
        tcb_signature = encode_dss_signature(r, s)

        tcb_pubk.verify(
            tcb_signature,
            tcb_info_body,
            ec.ECDSA(hashes.SHA256())
        )

        # Checking the tcbStatus is supported
        cpusvn = [x for x in quote[48:64]]
        pcesvn = struct.unpack("<H", quote[10:12])[0]
        tcb_level = self._dcap_find_tcb_level(
            tcb_info_dic["tcbInfo"]["tcbLevels"], cpusvn, pcesvn
        )
        if tcb_level is None:
            raise Exception("TCB level not supported")
        self._dcap_check_status(spec_dcap, tcb_level["tcbStatus"])

        qe_id_dic = json.loads(dcap.qeIdentity)
        qe_id_body = bytes(
            json.dumps(qe_id_dic["enclaveIdentity"], separators=(",", ":")), "utf-8"
        )
        qe_id_sign = bytearray.fromhex(qe_id_dic["signature"])

        # Verify the Qe Sign Cert
        qe_cert = OpenSSL.crypto.load_certificate(
            OpenSSL.crypto.FILETYPE_PEM, dcap.qeSignCert
        )
        qe_cert_context = OpenSSL.crypto.X509StoreContext(
            trust_store, qe_cert, list(intermediate_certs)
        )
        qe_cert_context.verify_certificate()

        # Verify Signature over enclave identity
        qe_sign_cert = cryptography.x509.load_pem_x509_certificate(dcap.qeSignCert)
        qe_pubk = qe_sign_cert.public_key()
        if not isinstance(qe_pubk, ec.EllipticCurvePublicKey):
            raise Exception("Expected EC public key")

        # Convert the signature from raw format to DSS format
        r = int.from_bytes(qe_id_sign[:32], "big")
        s = int.from_bytes(qe_id_sign[32:], "big")
        qe_signature = encode_dss_signature(r, s)

        qe_pubk.verify(
            qe_signature,
            qe_id_body,
            ec.ECDSA(hashes.SHA256())
        )

        # Checking if isvprodid is matching
        isvprodid_qe = int.from_bytes(quote[820:822], "little")
        if qe_id_dic["enclaveIdentity"]["isvprodid"] != isvprodid_qe:
            raise Exception(
                f'QE identity of quote {isvprodid_qe} does not match expected identity {qe_id_dic["enclaveIdentity"]["isvprodid"]}'
            )

        isvsvn = int.from_bytes(quote[822:824], "little")

        # Cheking if tcbStatus of the QUoting enclave is supported
        qe_tcb_level = next(
            (
                level
                for level in qe_id_dic["enclaveIdentity"]["tcbLevels"]
                if level["tcb"]["isvsvn"] <= isvsvn
            ),
            None,
        )
        if qe_tcb_level is None:
            raise Exception("QE TCB level not supported")

        return quote

    def _verify_dcap(self, dcap: FatquoteDcap) -> bytes:
        spec_type = self.attestation_specification.WhichOneof(
            "attestation_specification"
        )
        if spec_type != "intelDcap":
            raise Exception(
                f"Incompatible attestation specification, expected DCAP, got {spec_type}"
            )
        spec_dcap = self.attestation_specification.intelDcap
        quote = self._verify_dcap_inner(dcap, spec_dcap)

        # Asserting mrenclave
        measurement = bytes(quote[112:144])  ## [48+64 : 48+64+32]
        if not (spec_dcap.mrenclave == measurement):
            raise MeasurementMismatchError(
                f"Wrong measurement, expected measurement: {spec_dcap.mrenclave.hex()}, actual measurement: {measurement.hex()}."
            )
        return quote[368:400]

    def _verify_dcap_mrsigner(self, dcap: FatquoteDcapMrsigner) -> bytes:
        if (
            self.attestation_specification.WhichOneof("attestation_specification")
            != "intelDcapMrsigner"
        ):
            raise Exception(f"Incompatible attestation specification, expected DCAP")
        spec_dcap = self.attestation_specification.intelDcapMrsigner
        quote = self._verify_dcap_inner(dcap, spec_dcap)

        ## Asserting mrsigner and isvprodid
        mrsigner = bytes(quote[176:208])  # [48+128:48+128+32]
        if not mrsigner == spec_dcap.mrsigner:
            raise MrsignerMismatchError(
                f"Wrong mrsigner, expected {spec_dcap.mrsigner.hex()}, actual mrsigner {mrsigner.hex()}"
            )
        isvprodid = int.from_bytes(quote[304:306], "little")  # [48+256:48+256+2]
        if not isvprodid == spec_dcap.isvprodid:
            raise MrsignerMismatchError(
                f"Wrong isvprodid, expected 0x{hex(spec_dcap.isvprodid)}, actual 0x{hex(isvprodid)}"
            )

        mrenclave = bytes(quote[112:144])
        try:
            verify_measurement_transparency(
                serialize_length_delimited(dcap.sigstoreEvidence),
                spec_dcap.sigstoreRootOlpcJson,
                mrenclave,
            )
        except Exception as e:
            raise TransparencyVerificationError(f"{e}")

        return quote[368:400]

    def verify(self, fatquote: Fatquote) -> bytes:
        """
        :param certificate:
            A byte string of the certificate

        :param message:
            A byte string of the data the signature is for

        :param signature:
            A byte string of the data the signature is for

        :raises:
            CertificateValidationError
            SignatureValidationError
            MessageParsingError
            ISVEnclaveQuoteStatusError
            EnclaveQuoteFlagsError
            MeasurementMismatchError

        :return:
            the QuoteBody inside the message
        """

        if self.check_known_root_ca:
            if fatquote.HasField("epid"):
                _, _, ias_root_ca = asn1crypto.pem.unarmor(intel_sgx_ias_root_ca)
                if ias_root_ca != fatquote.epid.iasRootCaDer:
                    raise Exception(
                        f"Intel IAS Root CA in fatquote does not match known value"
                    )
            elif fatquote.HasField("dcap"):
                _, _, dcap_root_ca = asn1crypto.pem.unarmor(intel_sgx_dcap_root_ca)
                if dcap_root_ca != fatquote.dcap.dcapRootCaDer:
                    raise Exception(
                        f"Intel DCAP Root CA in fatquote does not match known value"
                    )
            elif fatquote.HasField("dcapMrsigner"):
                _, _, dcap_root_ca = asn1crypto.pem.unarmor(intel_sgx_dcap_root_ca)
                if dcap_root_ca != fatquote.dcapMrsigner.dcapRootCaDer:
                    raise Exception(
                        f"Intel DCAP Root CA in fatquote does not match known value"
                    )
            else:
                raise Exception(f"Unknown fatquote type")
        else:
            logger.warning(
                "!!!WARNING!!! MOCKED QUOTE is being accepted, quote is NOT to be trusted !!!WARNING!!!"
            )

        if fatquote.HasField("epid"):
            return self._verify_epid(fatquote.epid)
        elif fatquote.HasField("dcap"):
            return self._verify_dcap(fatquote.dcap)
        elif fatquote.HasField("dcapMrsigner"):
            return self._verify_dcap_mrsigner(fatquote.dcapMrsigner)
        else:
            raise Exception(f"Unknown fatquote type")



from __future__ import annotations
from typing import List
from typing_extensions import Self
from .participant import Participant


class RemarketingAudienceDefinition:
    """
    Class representing the definition of a remarketing audience.
    """

    def __init__(
        self,
        *,
        name: str,
        source_audience_type: str,
        shared_with: List[Participant],
    ) -> None:
        """
        Initialize a remarketing audience definition.

        **Parameters**:
        - `name`: Name of the remarketing audience
        - `source_audience_type`: The audience type of the seed audience to build from
        - `shared_with`: List of participant groups to share with
        """
        self.name = name
        self.source_audience_type = source_audience_type
        self.kind = "REMARKETING"
        self.shared_with = [participant.id for participant in shared_with]


class RemarketingAudienceBuilder:
    """
    Builder for constructing remarketing audience definitions.
    """

    def __init__(
        self,
        *,
        name: str,
        source_audience_type: str,
    ) -> None:
        """
        Initialise the remarketing audience builder.

        **Parameters**:
        - `name`: Name of the remarketing audience.
        - `source_audience_type`: The audience type of the seed audience that the remarketing audience will be built from.
        """
        self.name = name
        self.source_audience_type = source_audience_type
        self.shared_with = []

    def with_share_with_participants(self, participants: List[Participant]) -> Self:
        """
        Make the remarketing audience available to the given participants.

        **Parameters**:
        - `participants`: The participants to share the remarketing audience with

        **Returns**:
        - The updated remarketing audience builder
        """
        self.shared_with = participants
        return self

    def build(self) -> RemarketingAudienceDefinition:
        """
        Build the remarketing audience definition.

        **Returns**:
        - The remarketing audience definition
        """
        return RemarketingAudienceDefinition(
            name=self.name,
            source_audience_type=self.source_audience_type,
            shared_with=self.shared_with,
        )



"""Action classes for performing operations in a Media DCR."""

from ..client import Client
from ..types import JSONType
from decentriq_dcr_compiler.schemas.data_room import DataRoomAction
from .lookalike_audience_builder import LookalikeAudienceDefinition
from .rule_based_builder import RuleBasedAudienceDefinition
from .remarketing_audience_builder import RemarketingAudienceDefinition
from .api import (
    SeedAudience,
    CustomAudience,
)


class Action:
    def __init__(self, dcr_id: str, client: Client):
        """
        Initialize an Action instance.

        **Parameters**:
        - `dcr_id`: The identifier for the DCR
        - `client`: The client instance for API communication
        """
        self.dcr_id = dcr_id
        self.client = client
        self.session = client.create_session_v2()

    def _send_action(self, action: DataRoomAction) -> JSONType:
        """
        Send an action to the driver.

        **Parameters**:
        - `action`: The action to send to the driver

        **Returns**:
        - The response from the driver
        """
        return self.session.send_data_room_state_action_request(
            self.dcr_id, action.model_dump()
        )


class CreateLookalikeAudienceAction(Action):
    def __init__(
        self,
        dcr_id: str,
        client: Client,
        definition: LookalikeAudienceDefinition,
    ):
        """
        Initialize a CreateLookalikeAudienceAction instance.

        **Parameters**:
        - `dcr_id`: The identifier for the DCR
        - `client`: The client instance for API communication
        - `definition`: The definition of the lookalike audience to create
        """
        super().__init__(dcr_id, client)
        self.definition = definition

    def create(self) -> JSONType:
        """
        Create the lookalike audience.

        **Returns**:
        - The response from creating the lookalike audience
        """
        if isinstance(self.definition.source_audience, SeedAudience):
            source_ref_definition = {
                "kind": "SEED",
                "audienceType": self.definition.source_audience.audienceType,
            }
        elif isinstance(self.definition.source_audience, CustomAudience):
            source_ref_definition = {
                "id": self.definition.source_audience.id,
                "kind": "CUSTOM",
            }
        else:
            raise Exception(
                f"Invalid source audience type: {type(self.definition.source_audience)}"
            )

        definition = {
            "kind": "media",
            "createCustomAudience": {
                "definition": {
                    "kind": self.definition.kind,
                    "reach": self.definition.reach,
                    "sourceRef": source_ref_definition,
                    "excludeSeedAudience": self.definition.exclude_seed_audience,
                },
                "name": self.definition.name,
                "sharedWith": self.definition.shared_with,
            },
        }
        return self._send_action(DataRoomAction.model_validate(definition))


class CreateRuleBasedAudienceAction(Action):
    def __init__(
        self,
        dcr_id: str,
        client: Client,
        definition: RuleBasedAudienceDefinition,
    ):
        """
        Initialize a CreateRuleBasedAudienceAction instance.

        **Parameters**:
        - `dcr_id`: The identifier for the DCR
        - `client`: The client instance for API communication
        - `definition`: The definition of the rule based audience to create
        """
        super().__init__(dcr_id, client)
        self.definition = definition

    def create(self) -> JSONType:
        """
        Create the rule based audience.

        **Returns**:
        - The response from creating the rule based audience
        """
        if isinstance(self.definition.source_audience, SeedAudience):
            source_ref_definition = {
                "kind": "SEED",
                "audienceType": self.definition.source_audience.audienceType,
            }
        elif isinstance(self.definition.source_audience, CustomAudience):
            source_ref_definition = {
                "id": self.definition.source_audience.id,
                "kind": "CUSTOM",
            }
        else:
            raise Exception(
                f"Invalid source audience type: {type(self.definition.source_audience)}"
            )

        definition = {
            "kind": "media",
            "createCustomAudience": {
                "definition": {
                    "combine": (
                        [
                            combinator.as_dict()
                            for combinator in self.definition.combinators
                        ] if self.definition.combinators else None
                    ),
                    "filters": (
                        self.definition.filters.as_dict()
                        if self.definition.filters else None
                    ),
                    "kind": self.definition.kind,
                    "sourceRef": source_ref_definition,
                },
                "name": self.definition.name,
                "sharedWith": self.definition.shared_with,
            },
        }
        return self._send_action(DataRoomAction.model_validate(definition))


class CreateRemarketingAudienceAction(Action):
    def __init__(
        self,
        dcr_id: str,
        client: Client,
        definition: RemarketingAudienceDefinition,
    ):
        """
        Initialize a CreateRemarketingAudienceAction instance.

        **Parameters**:
        - `dcr_id`: The identifier for the DCR
        - `client`: The client instance for API communication
        - `definition`: The definition of the remarketing audience to create
        """
        super().__init__(dcr_id, client)
        self.definition = definition

    def create(self) -> JSONType:
        """
        Create the remarketing audience.

        **Returns**:
        - The response from creating the remarketing audience
        """
        source_ref_definition = {
            "audienceType": self.definition.source_audience_type,
        }

        definition = {
            "kind": "media",
            "createCustomAudience": {
                "definition": {
                    "kind": self.definition.kind,
                    "seedAudienceRef": source_ref_definition,
                },
                "name": self.definition.name,
                "sharedWith": self.definition.shared_with,
            },
        }
        return self._send_action(DataRoomAction.model_validate(definition))


class UpdateCustomAudienceAction(Action):
    def __init__(
        self,
        dcr_id: str,
        client: Client,
        audience: CustomAudience,
    ):
        """
        Initialize an UpdateCustomAudienceAction instance.

        **Parameters**:
        - `dcr_id`: The identifier for the DCR
        - `client`: The client instance for API communication
        - `audience`: The audience to update
        """
        super().__init__(dcr_id, client)
        self.audience = audience

    def update(self) -> JSONType:
        """
        Update the custom audience.

        **Returns**:
        - The response from updating the custom audience
        """
        definition = {
            "kind": "media",
            "updateCustomAudience": {
                "audienceId": self.audience.id,
                "definition": self.audience.definition.model_dump(),
                "name": self.audience.name,
                "sharedWith": self.audience.sharedWith,
            },
        }
        return self._send_action(DataRoomAction.model_validate(definition))


class GetAudiencePrerequisitesAction(Action):
    def __init__(self, dcr_id: str, client: Client, audience: CustomAudience):
        super().__init__(dcr_id, client)
        self.audience = audience

    def get(self) -> JSONType:
        definition = {
            "kind": "media",
            "getAudiencePrerequisites": {"audienceId": self.audience.id},
        }
        return self._send_action(DataRoomAction.model_validate(definition))


class DeleteCustomAudienceAction(Action):
    def __init__(
        self,
        dcr_id: str,
        client: Client,
        audience_id: str,
        force_delete_prerequisites: bool,
    ):
        """
        Initialize a DeleteCustomAudienceAction instance.

        **Parameters**:
        - `dcr_id`: The identifier for the DCR
        - `client`: The client instance for API communication
        - `audience_id`: The identifier for the audience to delete
        - `force_delete_prerequisites`: If true, the audience and all dependent audiences will be deleted. If false, the audience will only be deleted if it has no dependents.
        """
        super().__init__(dcr_id, client)
        self.audience_id = audience_id
        self.force_delete_prerequisites = force_delete_prerequisites

    def delete(self) -> JSONType:
        definition = {
            "kind": "media",
            "deleteCustomAudience": {
                "audienceId": self.audience_id,
                "forceDeletePrerequisites": self.force_delete_prerequisites,
            },
        }
        return self._send_action(DataRoomAction.model_validate(definition))



import csv
import io
import zipfile
from typing import List, Tuple

from ..proto.compute_sql_pb2 import TableSchema
from ..proto.length_delimited import parse_length_delimited


def read_input_csv_file(
    path: str,
    /,
    *,
    has_header: bool = True,
    check_header: bool = True,
    encoding="utf-8",
    **kwargs,
) -> io.BytesIO:
    """
    Read CSV from a file and turn it into a bytes array of the correct format so that it can be uploaded
    to the Decentriq platform.

    **Parameters**:
    - `path`: The path to the CSV file.
    - `has_header`: Whether the string contains a header row.
    - `check_header`: Whether the function should try to determine whether the file has a header.
        If the file has a header row but you didn't set the `has_header` flag, an
        exception will be raised. If you're sure that the way you use the function
        is correct, you can disable this check using this parameter.
    - `encoding`: The encoding of the CSV file. If you wrote the CSV file using a library like pandas,
        you need to check the documentation to see what encoding they use by default
        when writing files (likely `"utf-8"` in which case this can be left at its default value).
    - `delimiter`: What delimiter is used in the the CSV file. Default is the comma.
    - `**kwargs`: Additional keyword arguments passed to the Python CSV parser. Refer to the
        [official documentation](https://docs.python.org/3/library/csv.html#csv-fmt-params)
        for a list of supported arguments.

    **Returns**:
    A BytesIO object that can be passed to the methods resposible for uploading data.
    """
    with open(path, "r", encoding=encoding) as csvfile:
        return _read_input_csv(
            csvfile,
            has_header=has_header,
            check_header=check_header,
            **kwargs,
        )


def read_input_csv_string(
    content: str, /, *, has_header: bool = True, check_header: bool = True, **kwargs
) -> io.BytesIO:
    """
    Read CSV from a string and turn it into a bytes array of the correct format so that it can be uploaded
    to the Decentriq platform.

    **Parameters**:
    - `content`: The string containing the CSV file.
    - `has_header`: Whether the string contains a header row.
    - `check_header`: Whether the function should try to determine whether the file has a header.
        If the file has a header row but you didn't set the `has_header` flag, an
        exception will be raised. If you're sure that the way you use the function
        is correct, you can disable this check using this parameter.
    - `encoding`: The encoding of the source file. If you wrote the CSV file using a library like pandas,
        you need to check the documentation to see what encoding they use by default
        when writing files.
    - `delimiter`: What delimiter is used in the the CSV file. Default is the comma.
    - `**kwargs`: Additional keyword arguments passed to the Python CSV parser. What flags can be passed can be
        seen [here](https://docs.python.org/3/library/csv.html#csv-fmt-params).

    **Returns**:
    A BytesIO object that can be passed to the methods resposible for uploading data.
    """
    return _read_input_csv(
        io.StringIO(content.strip()),
        has_header=has_header,
        check_header=check_header,
        **kwargs,
    )


def _read_input_csv(
    data: io.TextIOBase,
    /,
    *,
    has_header: bool = True,
    check_header: bool = True,
    **kwargs,
) -> io.BytesIO:
    if check_header:
        sample = "\n".join(data.readline() for _ in range(20))
        file_has_header = csv.Sniffer().has_header(sample)
        if file_has_header and has_header is False:
            raise Exception(
                "Warning: the file seems to have a header but the header flag is not set to true!"
                " This flag needs to be set correctly in order for the data to be uploaded."
            )
        elif not file_has_header and has_header is True:
            raise Exception(
                "Warning: the file doesn't seem to have a header but the header flag is set!"
                " This flag needs to be set correctly in order for the data to be uploaded."
            )
        data.seek(0)
    reader = csv.reader(data, **kwargs)
    output = io.StringIO(newline="")
    writer = csv.writer(
        output, delimiter=",", quotechar='"', quoting=csv.QUOTE_MINIMAL, dialect="unix"
    )
    if has_header:
        next(reader)
    # This will write the full file into memory and encode it in bulk.
    # Could be wrapped in a IO buffer-style class that would perform the
    # read/write csv lines operation in a streaming fashion.
    for row in reader:
        writer.writerow(row)
    output.seek(0)
    return io.BytesIO(output.read().encode("utf-8"))


def _read_sql_query_result(result: bytes) -> Tuple[str, TableSchema]:
    archive = zipfile.ZipFile(io.BytesIO(result), "r")

    if not {"dataset.csv", "types"}.issubset(archive.namelist()):
        raise Exception(
            "The given result cannot be read as it doesn't contain all the required files."
            " Expected files: 'dataset.csv' and 'types', only found: [{}]".format(
                ", ".join(archive.namelist())
            )
        )

    csv = archive.read("dataset.csv").decode()
    schema = TableSchema()
    parse_length_delimited(archive.read("types"), schema)

    return (csv, schema)


def read_sql_query_result_as_list(result: bytes) -> Tuple[List[List[str]], TableSchema]:
    """
    Read the given raw CSV output from an SQL query result and transform it into a list of
    lists, where each inner list corresponds to a row in the tabular output data.
    The header row won't be contained in the resulting output and all the inner values
    will be of type string.
    The actual column types as well as their names will be returned as part of the schema
    value.
    """
    csv_content, schema = _read_sql_query_result(result)
    reader = csv.reader(io.StringIO(csv_content))
    return list(reader), schema


def read_sql_query_result_as_string(result: bytes, include_header: bool = True) -> str:
    """
    Read the given raw CSV output from an SQL query result and transform it into a string
    containing the full CSV file including the header row.
    The resulting string can be written directly to a file from where it can be read
    using your data analysis library of choice.

    **Parameters**:
    - `result`: The result from which to extract a CSV file as a string.
    - `include_header`: Whether to include an additional header row that contains
        column names (provided for example by using alias expressions
        in SELECT statements). If this setting is true, but no column
        names could be found, the names "V1", "V2" and so will be used.
    """
    content, schema = _read_sql_query_result(result)

    if include_header:
        opt_columns = [col.name for col in schema.namedColumns]
        # Use V1, V2, ... for unknown column names => same format as R uses.
        replacement_columns = [f"V{ix}" for ix in range(1, len(opt_columns) + 1)]
        columns = [c1 if c1 else c2 for c1, c2 in zip(opt_columns, replacement_columns)]
        returned_content = ",".join(columns) + "\n" + content
    else:
        returned_content = content

    # Remove trailing newlines
    return returned_content.strip()



import uuid
from typing import Any, Dict, List, Optional
from decentriq_dcr_compiler.schemas import Permission


class Participant:
    """
    A participant in a Data Clean Room.
    """

    def __init__(
        self,
        emails: List[str],
        permissions: List[Permission],
        role: str,
        id: Optional[str] = None,
        organization_id: Optional[str] = None,
        member_permissions: Optional[Dict[str, List[Permission]]] = None,
    ):
        """
        Initialize a participant.

        **Parameters**:
        - `emails`: The emails of the participant
        - `permissions`: The permissions of the participant
        - `role`: The role of the participant
        - `id`: The id of the participant
        - `organization_id`: The organization id of the participant
        - `member_permissions`: Specific permissions for the participant
        """
        self.id = str(uuid.uuid4()) if id is None else id
        self.role = role
        self.emails = emails
        self.permissions = permissions
        self.organization_id = organization_id
        self.member_permissions = member_permissions

    def as_dict(self) -> Dict[str, Any]:
        """
        Return the participant as a dictionary.

        **Returns**:
        - The participant as a dictionary
        """
        if self.member_permissions is None:
            member_permissions = None
        else:
            member_permissions = {
                email: [permission.value for permission in permissions]
                for email, permissions in self.member_permissions.items()
            }
        return {
            "id": self.id,
            "role": self.role,
            "emails": self.emails,
            "permissions": [permission.value for permission in self.permissions],
            "organizationId": self.organization_id,
            "memberPermissions": member_permissions,
        }

    def can_export_audience(self) -> bool:
        """
        Check if the participant has the export audience permission.

        **Returns**:
        - True if the participant has the export audience permission, False otherwise
        """
        return Permission.EXPORT_AUDIENCE.value in self.permissions



from typing import List
from google.protobuf.json_format import MessageToDict

from ...proto import (
    ComputeNodeFormat,
    serialize_length_delimited,
)
from ..node import Node
from .proto import AdformDspWorkerConfiguration, SinkInput


class AdformDspDataTransformer(Node):
    """
    Compute node that transforms data into an Adform compliant form.
    """

    def __init__(
        self,
        name: str,
        segment_owners: List[str],
        input: SinkInput,
    ) -> None:
        config = AdformDspWorkerConfiguration(
            input=input,
            segment_owners=segment_owners,
        )
        config_serialized = serialize_length_delimited(config)
        super().__init__(
            name,
            config=config_serialized,
            enclave_type="decentriq.adform-dsp-worker",
            dependencies=[input.dependency],
            output_format=ComputeNodeFormat.RAW,
        )


from ...decoder import AdformDspWorkerDecoder

__all__ = [
    "AdformDspDataTransformer",
    "AdformDspWorkerDecoder",
]



from datetime import datetime
import uuid


def generate_id() -> str:
    return str(uuid.uuid4())

def current_iso_standard_utc_time() -> str:
    return str(datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%f"))[:-3] + "Z"


from __future__ import annotations

import base64
import functools
import uuid
from enum import Enum
from typing import TYPE_CHECKING, Dict, List, Optional

from typing_extensions import Self

from ..attestation import enclave_specifications
from ..proto.length_delimited import serialize_length_delimited
from ..session import LATEST_WORKER_PROTOCOL_VERSION
from ..types import EnclaveSpecification
from .analytics_dcr import AnalyticsDcrDefinition
from .node_definitions import NodeDefinition
from .version import DATA_SCIENCE_DCR_SUPPORTED_VERSION

if TYPE_CHECKING:
    from ..client import Client


__all__ = [
    "AnalyticsDcrBuilder",
]


class AnalyticsDcrType(str, Enum):
    STATIC = "STATIC"
    INTERACTIVE = "INTERACTIVE"


class ParticipantPermission(Enum):
    DATA_OWNER = 1
    ANALYST = 2


def _get_hl_specs(enclave_specs: Dict[str, EnclaveSpecification]):
    specs = [
        {
            "attestationProtoBase64": base64.b64encode(
                serialize_length_delimited(spec["proto"])
            ).decode(),
            "id": name,
            "workerProtocol": spec.get(
                "workerProtocol", LATEST_WORKER_PROTOCOL_VERSION
            ),
        }
        for name, spec in enclave_specs.items()
    ]
    return specs


class AnalyticsDcrBuilder:
    """
    A builder for constructing Analytics Data Clean Rooms.
    """

    def __init__(
        self,
        *,
        client: Client,
        enclave_specs: Optional[Dict[str, EnclaveSpecification]] = None,
    ) -> None:
        """
        Initialise an Analytics DCR builder.

        **Parameters**:
        - `client`: A `Client` object that can be used to retrieve information about the platform.
        - `enclave_specs`: Determines the types of enclaves that will supported by this Data Clean Room.
            If not specified, the latest enclave specifications known to this
            SDK version will be used.
        """
        self.client = client
        self.enclave_specs = (
            enclave_specs if enclave_specs else enclave_specifications.latest()
        )
        self.name = None
        self.description = ""
        self.owner = None
        self.dcr_id = None
        self.compile_context = None
        self.node_definitions = []
        """The current list of Node Definitions that will be added to the Data Clean Room."""
        self.permissions = []
        """The list of permissions that will be added to the Data Clean Room."""
        self._force_spark_validation = False
        """Whether to force validation pipelines to use spark."""

    def _with_force_spark_validation(self, use_spark: bool) -> Self:
        """
        Whether to force the any validation pipeline part of this DCR
        to use spark. If this is false, the validation pipeline will decide
        based on the file size whether to use spark or not.

        **Parameters**:
        - `use_spark`: Force validation to use spark.
        """
        self._force_spark_validation = use_spark
        return self

    def with_name(self, name: str) -> Self:
        """
        Set the name of the Data Clean Room.

        **Parameters**:
        - `name`: Name to be used for the Data Clean Room.
        """
        self.name = name
        return self

    def with_description(self, description: str) -> Self:
        """
        Set the description of the Data Clean Room.

        **Parameters**:
        - `description`: Description of the Data Clean Room.
        """
        self.description = description
        return self

    def add_participant(
        self,
        email: str,
        *,
        analyst_of: List[str] = [],
        data_owner_of: List[str] = [],
    ) -> Self:
        """
        Add a participant to the DCR being built.

        If the participant isn't assigned a role, the user can still view the
        DCR but cannot interact with it.

        **Parameters**:
        - `email`: The email address of the participant.
        - `analyst_of`: The names of the Compute Nodes that the user can run.
        - `data_owner_of`: The names of the Data Nodes to which the user can
          connect a dataset.
        """
        if any(permissions["user"] == email for permissions in self.permissions):
            raise Exception(f"Participant with email {email} has already been added.")

        anaylst_permissions = [{"analyst": {"nodeId": node}} for node in analyst_of]
        data_owner_permissions = [
            {"dataOwner": {"nodeId": node}} for node in data_owner_of
        ]
        self.permissions.append(
            {"user": email, "permissions": anaylst_permissions + data_owner_permissions}
        )
        return self

    def with_owner(self, email: str) -> Self:
        """
        Set the owner of the Data Clean Room.

        **Parameters**:
        - `email`: The email address of the owner of the Data Clean Room.
        """
        self.owner = email
        return self

    def add_node_definition(self, definition: NodeDefinition) -> Self:
        """
        Add a single node definition to this builder.

        A node definition defines how a Compute or Data Node
        should be constructed.
        """
        self.add_node_definitions([definition])
        return self

    def add_node_definitions(self, definitions: List[NodeDefinition]) -> Self:
        """
        Add a list of node definitions to this builder.

        Each node definition defines how the respective Compute or Data Node
        should be constructed.
        """
        self.node_definitions.extend(definitions)
        return self

    def build(self) -> AnalyticsDcrDefinition:
        """
        Build the Data Clean Room.

        In order to use the DCR, the output of this method should be passed to
        `client.publish_analytics_dcr`.
        """
        if not self.owner:
            raise Exception("The Data Room owner must be specified")
        if not self.name:
            raise Exception("The Data Room name must be specified")

        nodes = [
            node._get_high_level_representation() for node in self.node_definitions
        ]
        required_workers = functools.reduce(
            lambda a, b: a.union(b),
            [set(node.required_workers) for node in self.node_definitions],
            {"decentriq.driver"},
        )
        used_enclave_specs = {}
        for worker in required_workers:
            if worker in self.enclave_specs:
                used_enclave_specs[worker] = self.enclave_specs[worker]
            else:
                raise Exception(
                    f"One of the nodes you added requires a worker of type '{worker}',"
                    " but no enclave specification matching this worker is known to this builder."
                )
        used_hl_enclave_specs = _get_hl_specs(used_enclave_specs)
        permissions = self._add_owner_permissions()
        self._check_correct_node_permissions(permissions)
        hl_dcr = {
            DATA_SCIENCE_DCR_SUPPORTED_VERSION: {
                # Only interactive DCRs are supported.
                "interactive": {
                    "commits": [],
                    "enableAutomergeFeature": True,
                    "initialConfiguration": {
                        "description": self.description,
                        "enableAirlock": True,
                        "enableAllowEmptyFilesInValidation": True,
                        "enableDevelopment": True,
                        "enablePostWorker": True,
                        "enableSafePythonWorkerStacktrace": True,
                        "enableServersideWasmValidation": True,
                        "enableSqliteWorker": True,
                        "enableTestDatasets": True,
                        "enclaveRootCertificatePem": self.client.decentriq_ca_root_certificate.decode(),
                        "enclaveSpecifications": used_hl_enclave_specs,
                        "enableForceSparkValidation": self._force_spark_validation,
                        "id": self._generate_id(),
                        "nodes": nodes,
                        "participants": permissions,
                        "title": self.name,
                    },
                }
            }
        }
        return AnalyticsDcrDefinition(
            name=self.name, high_level=hl_dcr, enclave_specs=self.enclave_specs
        )

    def _add_owner_permissions(self):
        for entry in self.permissions:
            if entry["user"] == self.owner:
                permissions = entry["permissions"]
                permissions.append({"manager": {}})
                entry["permissions"] = permissions
                return self.permissions

        # Entry wasn't found for existing user, so add a new one.
        self.permissions.append({"user": self.owner, "permissions": [{"manager": {}}]})
        return self.permissions

    def _check_correct_node_permissions(self, permissions: List[Dict]):
        node_lookup = {node.name: node for node in self.node_definitions}
        for permission in permissions:
            for permission_entry in permission["permissions"]:
                if "dataOwner" in permission_entry:
                    node_name = permission_entry["dataOwner"]["nodeId"]
                    if node_name not in node_lookup:
                        raise Exception(f"Data owner permission added for non-existent node {node_name}")
                    if not node_lookup[node_name].is_data_node():
                        raise Exception(f"Data owner permission cannot be added for non-data node {node_name}")
                elif "analyst" in permission_entry:
                    node_name = permission_entry["analyst"]["nodeId"]
                    if node_name not in node_lookup:
                        raise Exception(f"Analyst permission added for non-existent node {node_name}")
                    if not node_lookup[node_name].is_computation_node():
                        raise Exception(f"Analyst permission cannot be added for non-compute node {node_name}")


    @staticmethod
    def _generate_id():
        return str(uuid.uuid4())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: metering.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'metering.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import attestation_pb2 as attestation__pb2
from . import data_room_pb2 as data__room__pb2
from . import integrity_store_pb2 as integrity__store__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0emetering.proto\x12\x08metering\x1a\x11\x61ttestation.proto\x1a\x0f\x64\x61ta_room.proto\x1a\x15integrity_store.proto\"[\n\x0b\x45nclaveInfo\x12\x1e\n\x16\x61ttestationSpecHashHex\x18\x01 \x01(\t\x12\x1a\n\rtaskQueueName\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x10\n\x0e_taskQueueName\"\xc7\x08\n\x0fMeteringRequest\x12/\n\tcreateDcr\x18\x01 \x01(\x0b\x32\x1a.metering.CreateDcrRequestH\x00\x12;\n\x0f\x63reateDcrCommit\x18\x02 \x01(\x0b\x32 .metering.CreateDcrCommitRequestH\x00\x12+\n\x07stopDcr\x18\x03 \x01(\x0b\x32\x18.metering.StopDcrRequestH\x00\x12\x39\n\x0epublishDataset\x18\x04 \x01(\x0b\x32\x1f.metering.PublishDatasetRequestH\x00\x12=\n\x10unpublishDataset\x18\x05 \x01(\x0b\x32!.metering.UnpublishDatasetRequestH\x00\x12\x39\n\x0eworkerMetadata\x18\x06 \x01(\x0b\x32\x1f.metering.WorkerMetadataRequestH\x00\x12O\n\x19submitWorkerExecutionTime\x18\x07 \x01(\x0b\x32*.metering.SubmitWorkerExecutionTimeRequestH\x00\x12\x39\n\x0e\x64\x63rInteraction\x18\x08 \x01(\x0b\x32\x1f.metering.DcrInteractionRequestH\x00\x12\x37\n\rcreateDataset\x18\t \x01(\x0b\x32\x1e.metering.CreateDatasetRequestH\x00\x12K\n\x17getOrCreateDatasetScope\x18\n \x01(\x0b\x32(.metering.GetOrCreateDatasetScopeRequestH\x00\x12\x39\n\x0emergeDcrCommit\x18\x0b \x01(\x0b\x32\x1f.metering.MergeDcrCommitRequestH\x00\x12S\n\x1bgetDataConnectorCredentials\x18\x0c \x01(\x0b\x32,.metering.GetDataConnectorCredentialsRequestH\x00\x12\x44\n\x0eintegrityStore\x18\r \x01(\x0b\x32*.integrity_store_api.IntegrityStoreRequestH\x00\x12]\n getAzureCommunicationCredentials\x18\x0e \x01(\x0b\x32\x31.metering.GetAzureCommunicationCredentialsRequestH\x00\x12K\n\x17getOrCreateDcrDataScope\x18\x0f \x01(\x0b\x32(.metering.GetOrCreateDcrDataScopeRequestH\x00\x12\x45\n\x14recordDcrInteraction\x18\x10 \x01(\x0b\x32%.metering.RecordDcrInteractionRequestH\x00\x42\t\n\x07request\"\xdd\x08\n\x17MeteringSuccessResponse\x12\x30\n\tcreateDcr\x18\x01 \x01(\x0b\x32\x1b.metering.CreateDcrResponseH\x00\x12<\n\x0f\x63reateDcrCommit\x18\x02 \x01(\x0b\x32!.metering.CreateDcrCommitResponseH\x00\x12,\n\x07stopDcr\x18\x03 \x01(\x0b\x32\x19.metering.StopDcrResponseH\x00\x12:\n\x0epublishDataset\x18\x04 \x01(\x0b\x32 .metering.PublishDatasetResponseH\x00\x12>\n\x10unpublishDataset\x18\x05 \x01(\x0b\x32\".metering.UnpublishDatasetResponseH\x00\x12:\n\x0eworkerMetadata\x18\x06 \x01(\x0b\x32 .metering.WorkerMetadataResponseH\x00\x12P\n\x19submitWorkerExecutionTime\x18\x07 \x01(\x0b\x32+.metering.SubmitWorkerExecutionTimeResponseH\x00\x12:\n\x0e\x64\x63rInteraction\x18\x08 \x01(\x0b\x32 .metering.DcrInteractionResponseH\x00\x12\x38\n\rcreateDataset\x18\t \x01(\x0b\x32\x1f.metering.CreateDatasetResponseH\x00\x12L\n\x17getOrCreateDatasetScope\x18\n \x01(\x0b\x32).metering.GetOrCreateDatasetScopeResponseH\x00\x12:\n\x0emergeDcrCommit\x18\x0b \x01(\x0b\x32 .metering.MergeDcrCommitResponseH\x00\x12Q\n\x18\x64\x61taConnectorCredentials\x18\x0c \x01(\x0b\x32-.metering.GetDataConnectorCredentialsResponseH\x00\x12\x45\n\x0eintegrityStore\x18\r \x01(\x0b\x32+.integrity_store_api.IntegrityStoreResponseH\x00\x12^\n getAzureCommunicationCredentials\x18\x0e \x01(\x0b\x32\x32.metering.GetAzureCommunicationCredentialsResponseH\x00\x12L\n\x17getOrCreateDcrDataScope\x18\x0f \x01(\x0b\x32).metering.GetOrCreateDcrDataScopeResponseH\x00\x12\x46\n\x14recordDcrInteraction\x18\x10 \x01(\x0b\x32&.metering.RecordDcrInteractionResponseH\x00\x42\n\n\x08response\"g\n\x10MeteringResponse\x12\x34\n\x07success\x18\x01 \x01(\x0b\x32!.metering.MeteringSuccessResponseH\x00\x12\x11\n\x07\x66\x61ilure\x18\x02 \x01(\tH\x00\x42\n\n\x08response\"\x98\x01\n\x0b\x44\x63rMetadata\x12+\n\x07purpose\x18\x01 \x01(\x0e\x32\x1a.metering.CreateDcrPurpose\x12\x1c\n\x14showOrganizationLogo\x18\x02 \x01(\x08\x12\x17\n\x0frequirePassword\x18\x03 \x01(\x08\x12%\n\x04kind\x18\x04 \x01(\x0e\x32\x17.metering.CreateDcrKind\"\xb9\x01\n\x10\x43reateDcrRequest\x12\r\n\x05idHex\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x1d\n\x15\x64riverAttestationHash\x18\x03 \x01(\t\x12\x19\n\x11participantEmails\x18\x04 \x03(\t\x12\x15\n\x08metadata\x18\x05 \x01(\x0cH\x00\x88\x01\x01\x12*\n\x0b\x65nclaveInfo\x18\x06 \x03(\x0b\x32\x15.metering.EnclaveInfoB\x0b\n\t_metadata\"~\n!ExecuteComputationNodeInteraction\x12\x0e\n\x06nodeId\x18\x01 \x01(\t\x12\x38\n\x0crateLimiting\x18\x02 \x01(\x0b\x32\x1d.data_room.RateLimitingConfigH\x00\x88\x01\x01\x42\x0f\n\r_rateLimiting\"[\n\x1d\x45xecuteComputationInteraction\x12:\n\x05nodes\x18\x01 \x03(\x0b\x32+.metering.ExecuteComputationNodeInteraction\"v\n\x19PublishDatasetInteraction\x12\x0e\n\x06nodeId\x18\x01 \x01(\t\x12\x38\n\x0crateLimiting\x18\x02 \x01(\x0b\x32\x1d.data_room.RateLimitingConfigH\x00\x88\x01\x01\x42\x0f\n\r_rateLimiting\"\xa2\x01\n\x12\x44\x63rInteractionKind\x12\x45\n\x12\x65xecuteComputation\x18\x01 \x01(\x0b\x32\'.metering.ExecuteComputationInteractionH\x00\x12=\n\x0epublishDataset\x18\x02 \x01(\x0b\x32#.metering.PublishDatasetInteractionH\x00\x42\x06\n\x04kind\"\x90\x01\n\x15\x44\x63rInteractionRequest\x12\x14\n\x0c\x64\x61taRoomHash\x18\x01 \x01(\t\x12\x1d\n\x15\x64riverAttestationHash\x18\x02 \x01(\t\x12\x0f\n\x07scopeId\x18\x03 \x01(\t\x12\x31\n\x0binteraction\x18\x04 \x01(\x0b\x32\x1c.metering.DcrInteractionKind\"y\n\x15PublishDatasetRequest\x12\x15\n\rcomputeNodeId\x18\x01 \x01(\t\x12\x14\n\x0cmanifestHash\x18\x02 \x01(\t\x12\x14\n\x0c\x64\x61taRoomHash\x18\x03 \x01(\t\x12\x1d\n\x15\x64riverAttestationHash\x18\x04 \x01(\t\"\x99\x02\n\x14\x43reateDatasetRequest\x12\x14\n\x0cmanifestHash\x18\x01 \x01(\t\x12\x15\n\x08manifest\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x0f\n\x07scopeId\x18\x03 \x01(\t\x12\x0c\n\x04name\x18\x04 \x01(\t\x12\x18\n\x0b\x64\x65scription\x18\x05 \x01(\tH\x01\x88\x01\x01\x12\x16\n\tsizeBytes\x18\x06 \x01(\x04H\x02\x88\x01\x01\x12\x17\n\nstatistics\x18\x07 \x01(\tH\x03\x88\x01\x01\x12\x1c\n\x0f\x64\x61tasetImportId\x18\x08 \x01(\tH\x04\x88\x01\x01\x42\x0b\n\t_manifestB\x0e\n\x0c_descriptionB\x0c\n\n_sizeBytesB\r\n\x0b_statisticsB\x12\n\x10_datasetImportId\"L\n\x1eGetOrCreateDatasetScopeRequest\x12\x19\n\x0cmanifestHash\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_manifestHash\"2\n\x1eGetOrCreateDcrDataScopeRequest\x12\x10\n\x08\x64\x63rIdHex\x18\x01 \x01(\t\"e\n\x17UnpublishDatasetRequest\x12\x15\n\rcomputeNodeId\x18\x01 \x01(\t\x12\x14\n\x0c\x64\x61taRoomHash\x18\x02 \x01(\t\x12\x1d\n\x15\x64riverAttestationHash\x18\x03 \x01(\t\"E\n\x0eStopDcrRequest\x12\x14\n\x0c\x64\x61taRoomHash\x18\x01 \x01(\t\x12\x1d\n\x15\x64riverAttestationHash\x18\x02 \x01(\t\"\x81\x01\n\x16\x43reateDcrCommitRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08\x64\x63rIdHex\x18\x02 \x01(\t\x12\x1d\n\x15\x64riverAttestationHash\x18\x03 \x01(\t\x12*\n\x0b\x65nclaveInfo\x18\x06 \x03(\x0b\x32\x15.metering.EnclaveInfo\"\x80\x01\n\x15MergeDcrCommitRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08\x64\x63rIdHex\x18\x02 \x01(\t\x12\x1d\n\x15\x64riverAttestationHash\x18\x03 \x01(\t\x12*\n\x0b\x65nclaveInfo\x18\x06 \x03(\x0b\x32\x15.metering.EnclaveInfo\"\x13\n\x11\x43reateDcrResponse\"-\n\x1fGetOrCreateDatasetScopeResponse\x12\n\n\x02id\x18\x01 \x01(\t\"-\n\x1fGetOrCreateDcrDataScopeResponse\x12\n\n\x02id\x18\x01 \x01(\t\"\x18\n\x16\x44\x63rInteractionResponse\"\x19\n\x17\x43reateDcrCommitResponse\"\x18\n\x16MergeDcrCommitResponse\"\x11\n\x0fStopDcrResponse\"\x18\n\x16PublishDatasetResponse\"#\n\x15\x43reateDatasetResponse\x12\n\n\x02id\x18\x01 \x01(\t\"\x1a\n\x18UnpublishDatasetResponse\"h\n\x15WorkerMetadataRequest\x12>\n\x0f\x61ttestationSpec\x18\x01 \x01(\x0b\x32%.attestation.AttestationSpecification\x12\x0f\n\x07scopeId\x18\x02 \x01(\t\"5\n\x16WorkerMetadataResponse\x12\x1b\n\x13maxExecutionSeconds\x18\x01 \x01(\x04\"\x91\x01\n SubmitWorkerExecutionTimeRequest\x12\x1c\n\x14\x65xecutionTimeSeconds\x18\x01 \x01(\r\x12>\n\x0f\x61ttestationSpec\x18\x02 \x01(\x0b\x32%.attestation.AttestationSpecification\x12\x0f\n\x07scopeId\x18\x03 \x01(\t\"#\n!SubmitWorkerExecutionTimeResponse\"\x14\n\x12SplickyCredentials\"$\n\x12MicrosoftDspMember\x12\x0e\n\x06member\x18\x01 \x01(\t\"\x13\n\x11OnelogCredentials\"\x16\n\x14\x41mazonAmcCredentials\"\xf1\x01\n\x0f\x43redentialsKind\x12/\n\x07splicky\x18\x01 \x01(\x0b\x32\x1c.metering.SplickyCredentialsH\x00\x12:\n\x12microsoftDspMember\x18\x02 \x01(\x0b\x32\x1c.metering.MicrosoftDspMemberH\x00\x12-\n\x06onelog\x18\x03 \x01(\x0b\x32\x1b.metering.OnelogCredentialsH\x00\x12\x33\n\tamazonAmc\x18\x04 \x01(\x0b\x32\x1e.metering.AmazonAmcCredentialsH\x00\x42\r\n\x0b\x63redentials\"M\n\"GetDataConnectorCredentialsRequest\x12\'\n\x04kind\x18\x01 \x01(\x0b\x32\x19.metering.CredentialsKind\":\n#GetDataConnectorCredentialsResponse\x12\x13\n\x0b\x63redentials\x18\x01 \x01(\t\")\n\'GetAzureCommunicationCredentialsRequest\"d\n(GetAzureCommunicationCredentialsResponse\x12\x10\n\x08\x65ndpoint\x18\x01 \x01(\t\x12\x11\n\taccessKey\x18\x02 \x01(\t\x12\x13\n\x0bsenderEmail\x18\x03 \x01(\t\"d\n\x1bRecordDcrInteractionRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\t\x12\x31\n\x0binteraction\x18\x02 \x01(\x0b\x32\x1c.metering.DcrInteractionKind\"\x1e\n\x1cRecordDcrInteractionResponse*`\n\x10\x43reateDcrPurpose\x12\x0c\n\x08STANDARD\x10\x00\x12\x0e\n\nVALIDATION\x10\x01\x12\x0f\n\x0b\x44\x41TA_IMPORT\x10\x02\x12\x0f\n\x0b\x44\x41TA_EXPORT\x10\x03\x12\x0c\n\x08\x44\x41TA_LAB\x10\x04*|\n\rCreateDcrKind\x12\n\n\x06\x45XPERT\x10\x00\x12\x0f\n\x0b\x44\x41TASCIENCE\x10\x01\x12\t\n\x05MEDIA\x10\x02\x12\x13\n\x0fLOOKALIKE_MEDIA\x10\x03\x12\x12\n\x0eMEDIA_INSIGHTS\x10\x04\x12\x0c\n\x08\x41\x42_MEDIA\x10\x05\x12\x0c\n\x08MEDIA_V3\x10\x06\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'metering_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_CREATEDCRPURPOSE']._serialized_start=5890
  _globals['_CREATEDCRPURPOSE']._serialized_end=5986
  _globals['_CREATEDCRKIND']._serialized_start=5988
  _globals['_CREATEDCRKIND']._serialized_end=6112
  _globals['_ENCLAVEINFO']._serialized_start=87
  _globals['_ENCLAVEINFO']._serialized_end=178
  _globals['_METERINGREQUEST']._serialized_start=181
  _globals['_METERINGREQUEST']._serialized_end=1276
  _globals['_METERINGSUCCESSRESPONSE']._serialized_start=1279
  _globals['_METERINGSUCCESSRESPONSE']._serialized_end=2396
  _globals['_METERINGRESPONSE']._serialized_start=2398
  _globals['_METERINGRESPONSE']._serialized_end=2501
  _globals['_DCRMETADATA']._serialized_start=2504
  _globals['_DCRMETADATA']._serialized_end=2656
  _globals['_CREATEDCRREQUEST']._serialized_start=2659
  _globals['_CREATEDCRREQUEST']._serialized_end=2844
  _globals['_EXECUTECOMPUTATIONNODEINTERACTION']._serialized_start=2846
  _globals['_EXECUTECOMPUTATIONNODEINTERACTION']._serialized_end=2972
  _globals['_EXECUTECOMPUTATIONINTERACTION']._serialized_start=2974
  _globals['_EXECUTECOMPUTATIONINTERACTION']._serialized_end=3065
  _globals['_PUBLISHDATASETINTERACTION']._serialized_start=3067
  _globals['_PUBLISHDATASETINTERACTION']._serialized_end=3185
  _globals['_DCRINTERACTIONKIND']._serialized_start=3188
  _globals['_DCRINTERACTIONKIND']._serialized_end=3350
  _globals['_DCRINTERACTIONREQUEST']._serialized_start=3353
  _globals['_DCRINTERACTIONREQUEST']._serialized_end=3497
  _globals['_PUBLISHDATASETREQUEST']._serialized_start=3499
  _globals['_PUBLISHDATASETREQUEST']._serialized_end=3620
  _globals['_CREATEDATASETREQUEST']._serialized_start=3623
  _globals['_CREATEDATASETREQUEST']._serialized_end=3904
  _globals['_GETORCREATEDATASETSCOPEREQUEST']._serialized_start=3906
  _globals['_GETORCREATEDATASETSCOPEREQUEST']._serialized_end=3982
  _globals['_GETORCREATEDCRDATASCOPEREQUEST']._serialized_start=3984
  _globals['_GETORCREATEDCRDATASCOPEREQUEST']._serialized_end=4034
  _globals['_UNPUBLISHDATASETREQUEST']._serialized_start=4036
  _globals['_UNPUBLISHDATASETREQUEST']._serialized_end=4137
  _globals['_STOPDCRREQUEST']._serialized_start=4139
  _globals['_STOPDCRREQUEST']._serialized_end=4208
  _globals['_CREATEDCRCOMMITREQUEST']._serialized_start=4211
  _globals['_CREATEDCRCOMMITREQUEST']._serialized_end=4340
  _globals['_MERGEDCRCOMMITREQUEST']._serialized_start=4343
  _globals['_MERGEDCRCOMMITREQUEST']._serialized_end=4471
  _globals['_CREATEDCRRESPONSE']._serialized_start=4473
  _globals['_CREATEDCRRESPONSE']._serialized_end=4492
  _globals['_GETORCREATEDATASETSCOPERESPONSE']._serialized_start=4494
  _globals['_GETORCREATEDATASETSCOPERESPONSE']._serialized_end=4539
  _globals['_GETORCREATEDCRDATASCOPERESPONSE']._serialized_start=4541
  _globals['_GETORCREATEDCRDATASCOPERESPONSE']._serialized_end=4586
  _globals['_DCRINTERACTIONRESPONSE']._serialized_start=4588
  _globals['_DCRINTERACTIONRESPONSE']._serialized_end=4612
  _globals['_CREATEDCRCOMMITRESPONSE']._serialized_start=4614
  _globals['_CREATEDCRCOMMITRESPONSE']._serialized_end=4639
  _globals['_MERGEDCRCOMMITRESPONSE']._serialized_start=4641
  _globals['_MERGEDCRCOMMITRESPONSE']._serialized_end=4665
  _globals['_STOPDCRRESPONSE']._serialized_start=4667
  _globals['_STOPDCRRESPONSE']._serialized_end=4684
  _globals['_PUBLISHDATASETRESPONSE']._serialized_start=4686
  _globals['_PUBLISHDATASETRESPONSE']._serialized_end=4710
  _globals['_CREATEDATASETRESPONSE']._serialized_start=4712
  _globals['_CREATEDATASETRESPONSE']._serialized_end=4747
  _globals['_UNPUBLISHDATASETRESPONSE']._serialized_start=4749
  _globals['_UNPUBLISHDATASETRESPONSE']._serialized_end=4775
  _globals['_WORKERMETADATAREQUEST']._serialized_start=4777
  _globals['_WORKERMETADATAREQUEST']._serialized_end=4881
  _globals['_WORKERMETADATARESPONSE']._serialized_start=4883
  _globals['_WORKERMETADATARESPONSE']._serialized_end=4936
  _globals['_SUBMITWORKEREXECUTIONTIMEREQUEST']._serialized_start=4939
  _globals['_SUBMITWORKEREXECUTIONTIMEREQUEST']._serialized_end=5084
  _globals['_SUBMITWORKEREXECUTIONTIMERESPONSE']._serialized_start=5086
  _globals['_SUBMITWORKEREXECUTIONTIMERESPONSE']._serialized_end=5121
  _globals['_SPLICKYCREDENTIALS']._serialized_start=5123
  _globals['_SPLICKYCREDENTIALS']._serialized_end=5143
  _globals['_MICROSOFTDSPMEMBER']._serialized_start=5145
  _globals['_MICROSOFTDSPMEMBER']._serialized_end=5181
  _globals['_ONELOGCREDENTIALS']._serialized_start=5183
  _globals['_ONELOGCREDENTIALS']._serialized_end=5202
  _globals['_AMAZONAMCCREDENTIALS']._serialized_start=5204
  _globals['_AMAZONAMCCREDENTIALS']._serialized_end=5226
  _globals['_CREDENTIALSKIND']._serialized_start=5229
  _globals['_CREDENTIALSKIND']._serialized_end=5470
  _globals['_GETDATACONNECTORCREDENTIALSREQUEST']._serialized_start=5472
  _globals['_GETDATACONNECTORCREDENTIALSREQUEST']._serialized_end=5549
  _globals['_GETDATACONNECTORCREDENTIALSRESPONSE']._serialized_start=5551
  _globals['_GETDATACONNECTORCREDENTIALSRESPONSE']._serialized_end=5609
  _globals['_GETAZURECOMMUNICATIONCREDENTIALSREQUEST']._serialized_start=5611
  _globals['_GETAZURECOMMUNICATIONCREDENTIALSREQUEST']._serialized_end=5652
  _globals['_GETAZURECOMMUNICATIONCREDENTIALSRESPONSE']._serialized_start=5654
  _globals['_GETAZURECOMMUNICATIONCREDENTIALSRESPONSE']._serialized_end=5754
  _globals['_RECORDDCRINTERACTIONREQUEST']._serialized_start=5756
  _globals['_RECORDDCRINTERACTIONREQUEST']._serialized_end=5856
  _globals['_RECORDDCRINTERACTIONRESPONSE']._serialized_start=5858
  _globals['_RECORDDCRINTERACTIONRESPONSE']._serialized_end=5888
# @@protoc_insertion_point(module_scope)



from datetime import datetime
import json
from typing import Any, Dict
import uuid
from decentriq_dcr_compiler.schemas import Audience6 as Audience, ParameterPayloads
from decentriq_dcr_compiler import ab_media as ab_media_compiler

def generate_id() -> str:
    return str(uuid.uuid4())


def current_iso_standard_utc_time() -> str:
    return str(datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%f"))[:-3] + "Z"


def flatten_mutable_fields(definition: Dict[str, Any]) -> Dict[str, Any]:
    """
    Flatten the mutable field so that all mutable variables are
    included in the parent level of the dictionary.
    """
    audiences = definition.pop("audiences")
    flattened_dict = definition

    flattened_audience = []
    for audience in audiences:
        if "mutable" in audience:
            mutable = audience.pop("mutable")
            for k, v in mutable.items():
                audience[k] = v
        flattened_audience.append(audience)

    flattened_dict["audiences"] = flattened_audience
    return flattened_dict


def get_parameter_payloads(
    audience_name: str, audiences_json: Dict[str, Any]
) -> ParameterPayloads:
    audiences = {
        a["mutable"]["name"]: Audience.parse_raw(json.dumps(a))
        for a in audiences_json["audiences"]
    }

    if audience_name not in audiences:
        raise Exception(
            f'Requested audience with name "{audience_name}" not in list of available audiences'
        )

    target_audience = audiences[audience_name].root
    parameter_payloads_str = ab_media_compiler.get_parameter_payloads(
        target_ref=target_audience.id, audiences=list(audiences.values())
    )
    parameter_payloads = ParameterPayloads.parse_raw(parameter_payloads_str)
    return parameter_payloads


def audience_depends_on_lookalike(
    audience_name: str, audiences_json: Dict[str, Any]
) -> bool:
    audiences = {
        a["mutable"]["name"]: Audience.parse_raw(json.dumps(a))
        for a in audiences_json["audiences"]
    }

    if audience_name not in audiences:
        raise Exception(
            f'Requested audience with name "{audience_name}" not in list of available audiences'
        )

    target_audience = audiences[audience_name].root
    return ab_media_compiler.does_audience_depend_on_lookalike_audience(
        target_ref=target_audience.id, audiences=audiences.values()
    )


def get_dependencies(
    target_audience_id: str, audiences: list[Any]
) -> list[str]:
    return ab_media_compiler.get_dependencies(
        target_ref=target_audience_id,
        audiences=[Audience.parse_raw(json.dumps(a)) for a in audiences]
    )



from typing import List
from ..logger import logger


class AbMediaDcrFeatures:
    def __init__(self, features: List[str]) -> None:
        self.features = features

    def has_enable_rate_limiting_on_publish_dataset(self) -> bool:
        return self._features_contain("ENABLE_RATE_LIMITING_ON_PUBLISH_DATASET")

    def has_enable_timer_report(self) -> bool:
        return self._features_contain("ENABLE_TIMER_REPORT")

    def has_enable_model_performance_evaluation(self) -> bool:
        return self._features_contain("ENABLE_MODEL_PERFORMANCE_EVALUATION")

    def has_enable_drop_invalid_rows(self) -> bool:
        return self._features_contain("ENABLE_DROP_INVALID_ROWS")

    def has_enable_debug_mode(self) -> bool:
        debug_enabled = self._features_contain("ENABLE_DEBUG_MODE")
        if debug_enabled:
            logger.warning("!!!WARNING!!!: DEBUG MODE IS ENABLED")
        return debug_enabled

    def has_enable_insights(self) -> bool:
        return self._features_contain("ENABLE_INSIGHTS")

    def has_enable_lookalike_audiences(self) -> bool:
        return self._features_contain("ENABLE_LOOKALIKE_AUDIENCES")

    def has_enable_rule_based_audiences(self) -> bool:
        return self._features_contain("ENABLE_RULE_BASED_AUDIENCES")

    def has_enable_remarketing(self) -> bool:
        return self._features_contain("ENABLE_REMARKETING")

    def has_enable_exclude_seed_audience(self) -> bool:
        return self._features_contain("ENABLE_EXCLUDE_SEED_AUDIENCE")

    def has_enable_hide_absolute_audience_sizes(self) -> bool:
        return self._features_contain("ENABLE_HIDE_ABSOLUTE_AUDIENCE_SIZES")

    def has_enable_data_partner(self) -> bool:
        return self._features_contain("ENABLE_DATA_PARTNER")

    def has_enable_advertiser_audience_download(self) -> bool:
        return self._features_contain("ENABLE_ADVERTISER_AUDIENCE_DOWNLOAD")

    def _features_contain(self, flag: str) -> bool:
        return flag in self.features



import io, json
from typing import BinaryIO

class AwsCredentials:
    def __init__(self, access_key: str, secret_key: str) -> None:
        self.access_key = access_key
        self.secret_key = secret_key

    def as_binary_io(self) -> BinaryIO:
        credentials = {
            "accessKey": self.access_key,
            "secretKey": self.secret_key,
        }
        return io.BytesIO(json.dumps(credentials).encode())


from __future__ import annotations

import json
from typing import TYPE_CHECKING, Dict, List, Optional

from decentriq_dcr_compiler.schemas import PreviewComputationNode
from typing_extensions import Self

from ..session import Session
from .high_level_node import ComputationNode
from .node_definitions import NodeDefinition

if TYPE_CHECKING:
    from ..client import Client


class PreviewComputeNodeDefinition(NodeDefinition):
    """
    Class representing a Preview (Airlock) Compute Node Definition.
    """

    def __init__(
        self,
        name: str,
        dependency: str,
        quota_bytes: Optional[int] = 0,
        id: Optional[str] = None,
    ) -> None:
        """
        Initialise a `PreviewComputeNodeDefinition`:

        **Parameters**:
        `name`: Name of the `PreviewComputeNodeDefinition`.
        `dependency`: Node that the `PreviewComputeNodeDefinition` depends on.
        `quota_bytes`: Threshold for amount of data that can be previewed.
        """
        super().__init__(name=name, id=id or name)
        self.quota_bytes = quota_bytes
        self.dependency = dependency

    def _get_high_level_representation(self) -> Dict[str, str]:
        """
        Retrieve the high level representation of the `PreviewComputeNodeDefinition`.
        """
        computation_node = {
            "id": self.id,
            "name": self.name,
            "kind": {
                "computation": {
                    "kind": {
                        "preview": {
                            "dependency": self.dependency,
                            "quotaBytes": self.quota_bytes,
                        }
                    }
                },
            },
        }
        return computation_node

    @classmethod
    def _from_high_level(cls, id: str, name: str, node: PreviewComputationNode) -> Self:
        """
        Instantiate a `PreviewComputeNodeDefinition` from its high level representation.

        **Parameters**:
        - `name`: Name of the `PreviewComputeNodeDefinition`.
        - `node`: Pydantic model of the `PreviewComputeNode`.
        """
        preview_node = json.loads(node.model_dump_json())
        return cls(
            id=id,
            name=name,
            dependency=preview_node["dependency"],
            quota_bytes=preview_node["quotaBytes"],
        )

    @property
    def required_workers(self):
        return ["decentriq.driver"]

    def build(
        self,
        dcr_id: str,
        node_definition: NodeDefinition,
        client: Client,
        session: Session,
    ) -> PreviewComputeNode:
        """
        Construct a PreviewComputeNode from the Node Definition.

        **Parameters**:
        - `dcr_id`: ID of the DCR the node is a member of.
        - `node_definition`: Definition of the Matching Node.
        - `client`: A `Client` object which can be used to perform operations such as uploading data
            and retrieving computation results.
        - `session`: The session with which to communicate with the enclave.
        """
        return PreviewComputeNode(
            name=self.name,
            dcr_id=dcr_id,
            dependency=self.dependency,
            client=client,
            session=session,
            node_definition=node_definition,
            quota_bytes=self.quota_bytes,
            id=self.id,
        )


class PreviewComputeNode(ComputationNode):
    """
    Class representing a Preview (Airlock) Computation node.
    """

    def __init__(
        self,
        id: str,
        name: str,
        dcr_id: str,
        dependency: str,
        client: Client,
        session: Session,
        node_definition: NodeDefinition,
        quota_bytes: Optional[int] = 0,
    ) -> None:
        """
        Initialise a `PreviewComputeNode`:

        **Parameters**:
        - `name`: Name of the `PreviewComputeNode`.
        - `dcr_id`: ID of the DCR this node is part of.
        - `dependency`: Node that the `PreviewComputeNode` depends on.
        - `client`: A `Client` object which can be used to perform operations such as uploading data
            and retrieving computation results.
        - `session`: The session with which to communicate with the enclave.o
        - `node_definition`: Definition of the Preview Node.
        - `quota_bytes`: Threshold for amount of data that can be previewed.
        - `id`: ID of the `PreviewComputeNode`.
        """
        super().__init__(
            name=name,
            client=client,
            session=session,
            dcr_id=dcr_id,
            id=id,
            definition=node_definition,
        )
        self.dependency = dependency
        self.quota_bytes = quota_bytes
        self.node_definition = node_definition

    def _get_computation_id(self) -> str:
        """
        Retrieve the ID of the node corresponding to the computation.
        """
        return self.id



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class SingleFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["name", b"name"]) -> None: ...

global___SingleFile = SingleFile

@typing.final
class RawFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RawFile = RawFile

@typing.final
class ZipFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SINGLEFILE_FIELD_NUMBER: builtins.int
    @property
    def singleFile(self) -> global___SingleFile: ...
    def __init__(
        self,
        *,
        singleFile: global___SingleFile | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["selection", b"selection", "singleFile", b"singleFile"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["selection", b"selection", "singleFile", b"singleFile"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["selection", b"selection"]) -> typing.Literal["singleFile"] | None: ...

global___ZipFile = ZipFile

@typing.final
class SinkInput(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEPENDENCY_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    RAW_FIELD_NUMBER: builtins.int
    ZIP_FIELD_NUMBER: builtins.int
    dependency: builtins.str
    name: builtins.str
    @property
    def raw(self) -> global___RawFile: ...
    @property
    def zip(self) -> global___ZipFile: ...
    def __init__(
        self,
        *,
        dependency: builtins.str = ...,
        name: builtins.str = ...,
        raw: global___RawFile | None = ...,
        zip: global___ZipFile | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["file", b"file", "raw", b"raw", "zip", b"zip"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dependency", b"dependency", "file", b"file", "name", b"name", "raw", b"raw", "zip", b"zip"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["file", b"file"]) -> typing.Literal["raw", "zip"] | None: ...

global___SinkInput = SinkInput

@typing.final
class GoogleDv360SinkWorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INPUT_FIELD_NUMBER: builtins.int
    CREDENTIALSDEPENDENCY_FIELD_NUMBER: builtins.int
    ADVERTISERID_FIELD_NUMBER: builtins.int
    DISPLAYNAME_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    MEMBERSHIPDURATIONDAYS_FIELD_NUMBER: builtins.int
    USERCONSENTGIVEN_FIELD_NUMBER: builtins.int
    credentialsDependency: builtins.str
    advertiserId: builtins.str
    displayName: builtins.str
    description: builtins.str
    membershipDurationDays: builtins.str
    userConsentGiven: builtins.bool
    @property
    def input(self) -> global___SinkInput: ...
    def __init__(
        self,
        *,
        input: global___SinkInput | None = ...,
        credentialsDependency: builtins.str = ...,
        advertiserId: builtins.str = ...,
        displayName: builtins.str = ...,
        description: builtins.str = ...,
        membershipDurationDays: builtins.str = ...,
        userConsentGiven: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["input", b"input"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["advertiserId", b"advertiserId", "credentialsDependency", b"credentialsDependency", "description", b"description", "displayName", b"displayName", "input", b"input", "membershipDurationDays", b"membershipDurationDays", "userConsentGiven", b"userConsentGiven"]) -> None: ...

global___GoogleDv360SinkWorkerConfiguration = GoogleDv360SinkWorkerConfiguration



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: salesforce.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'salesforce.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10salesforce.proto\x12\nsalesforce\"\x06\n\x04\x43ore\"\x10\n\x0eMarketingCloud\"s\n\nImportRole\x12 \n\x04\x63ore\x18\x01 \x01(\x0b\x32\x10.salesforce.CoreH\x00\x12\x34\n\x0emarketingCloud\x18\x02 \x01(\x0b\x32\x1a.salesforce.MarketingCloudH\x00\x42\r\n\x0bproductType\"2\n\nExportRole\x12$\n\x05input\x18\x01 \x01(\x0b\x32\x15.salesforce.SinkInput\"\x1a\n\nSingleFile\x12\x0c\n\x04name\x18\x01 \x01(\t\"\t\n\x07RawFile\"D\n\x07ZipFile\x12,\n\nsingleFile\x18\x01 \x01(\x0b\x32\x16.salesforce.SingleFileH\x00\x42\x0b\n\tselection\"}\n\tSinkInput\x12\x12\n\ndependency\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\"\n\x03raw\x18\x03 \x01(\x0b\x32\x13.salesforce.RawFileH\x00\x12\"\n\x03zip\x18\x04 \x01(\x0b\x32\x13.salesforce.ZipFileH\x00\x42\x06\n\x04\x66ile\"\xc6\x01\n\x1dSalesforceWorkerConfiguration\x12\x1d\n\x15\x63redentialsDependency\x18\x01 \x01(\t\x12,\n\nimportRole\x18\x02 \x01(\x0b\x32\x16.salesforce.ImportRoleH\x00\x12,\n\nexportRole\x18\x03 \x01(\x0b\x32\x16.salesforce.ExportRoleH\x00\x12\x11\n\tdomainUrl\x18\x04 \x01(\t\x12\x0f\n\x07\x61piName\x18\x05 \x01(\tB\x06\n\x04roleb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'salesforce_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_CORE']._serialized_start=32
  _globals['_CORE']._serialized_end=38
  _globals['_MARKETINGCLOUD']._serialized_start=40
  _globals['_MARKETINGCLOUD']._serialized_end=56
  _globals['_IMPORTROLE']._serialized_start=58
  _globals['_IMPORTROLE']._serialized_end=173
  _globals['_EXPORTROLE']._serialized_start=175
  _globals['_EXPORTROLE']._serialized_end=225
  _globals['_SINGLEFILE']._serialized_start=227
  _globals['_SINGLEFILE']._serialized_end=253
  _globals['_RAWFILE']._serialized_start=255
  _globals['_RAWFILE']._serialized_end=264
  _globals['_ZIPFILE']._serialized_start=266
  _globals['_ZIPFILE']._serialized_end=334
  _globals['_SINKINPUT']._serialized_start=336
  _globals['_SINKINPUT']._serialized_end=461
  _globals['_SALESFORCEWORKERCONFIGURATION']._serialized_start=464
  _globals['_SALESFORCEWORKERCONFIGURATION']._serialized_end=662
# @@protoc_insertion_point(module_scope)
